// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: dynamic_partitions_device_info.proto
// Protobuf C++ Version: 7.35.0

#ifndef dynamic_5fpartitions_5fdevice_5finfo_2eproto_2epb_2eh
#define dynamic_5fpartitions_5fdevice_5finfo_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_dynamic_5fpartitions_5fdevice_5finfo_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_dynamic_5fpartitions_5fdevice_5finfo_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_dynamic_5fpartitions_5fdevice_5finfo_2eproto;
}  // extern "C"
namespace android {
class DynamicPartitionsDeviceInfoProto;
struct DynamicPartitionsDeviceInfoProtoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DynamicPartitionsDeviceInfoProtoGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_globals_;
extern const ::google::protobuf::internal::ClassDataFull DynamicPartitionsDeviceInfoProto_class_data_;
#else
extern const DynamicPartitionsDeviceInfoProtoGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class DynamicPartitionsDeviceInfoProto_BlockDevice;
struct DynamicPartitionsDeviceInfoProto_BlockDeviceGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DynamicPartitionsDeviceInfoProto_BlockDeviceGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_BlockDevice_globals_;
extern const ::google::protobuf::internal::ClassDataFull DynamicPartitionsDeviceInfoProto_BlockDevice_class_data_;
#else
extern const DynamicPartitionsDeviceInfoProto_BlockDeviceGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_BlockDevice_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class DynamicPartitionsDeviceInfoProto_Group;
struct DynamicPartitionsDeviceInfoProto_GroupGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DynamicPartitionsDeviceInfoProto_GroupGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_Group_globals_;
extern const ::google::protobuf::internal::ClassDataFull DynamicPartitionsDeviceInfoProto_Group_class_data_;
#else
extern const DynamicPartitionsDeviceInfoProto_GroupGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_Group_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class DynamicPartitionsDeviceInfoProto_Partition;
struct DynamicPartitionsDeviceInfoProto_PartitionGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DynamicPartitionsDeviceInfoProto_PartitionGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_Partition_globals_;
extern const ::google::protobuf::internal::ClassDataFull DynamicPartitionsDeviceInfoProto_Partition_class_data_;
#else
extern const DynamicPartitionsDeviceInfoProto_PartitionGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_Partition_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class DynamicPartitionsDeviceInfoProto_SuperDevice;
struct DynamicPartitionsDeviceInfoProto_SuperDeviceGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DynamicPartitionsDeviceInfoProto_SuperDeviceGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_SuperDevice_globals_;
extern const ::google::protobuf::internal::ClassDataFull DynamicPartitionsDeviceInfoProto_SuperDevice_class_data_;
#else
extern const DynamicPartitionsDeviceInfoProto_SuperDeviceGlobalsTypeInternal DynamicPartitionsDeviceInfoProto_SuperDevice_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace android
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace android {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED DynamicPartitionsDeviceInfoProto_SuperDevice final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:android.DynamicPartitionsDeviceInfoProto.SuperDevice) */ {
 public:
  inline DynamicPartitionsDeviceInfoProto_SuperDevice() : DynamicPartitionsDeviceInfoProto_SuperDevice(nullptr) {}
  ~DynamicPartitionsDeviceInfoProto_SuperDevice() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DynamicPartitionsDeviceInfoProto_SuperDevice));
  }
#endif

  template <typename = void>
  explicit constexpr DynamicPartitionsDeviceInfoProto_SuperDevice(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline DynamicPartitionsDeviceInfoProto_SuperDevice(const DynamicPartitionsDeviceInfoProto_SuperDevice& from) : DynamicPartitionsDeviceInfoProto_SuperDevice(nullptr, from) {}
  inline DynamicPartitionsDeviceInfoProto_SuperDevice(DynamicPartitionsDeviceInfoProto_SuperDevice&& from) noexcept : DynamicPartitionsDeviceInfoProto_SuperDevice(nullptr, ::std::move(from)) {}
  inline DynamicPartitionsDeviceInfoProto_SuperDevice& operator=(const DynamicPartitionsDeviceInfoProto_SuperDevice& from) {
    CopyFrom(from);
    return *this;
  }
  inline DynamicPartitionsDeviceInfoProto_SuperDevice& operator=(DynamicPartitionsDeviceInfoProto_SuperDevice&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const DynamicPartitionsDeviceInfoProto_SuperDevice& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<DynamicPartitionsDeviceInfoProto_SuperDevice>(&DynamicPartitionsDeviceInfoProto_SuperDevice_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(DynamicPartitionsDeviceInfoProto_SuperDevice& a, DynamicPartitionsDeviceInfoProto_SuperDevice& b) { a.Swap(&b); }
  inline void Swap(DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DynamicPartitionsDeviceInfoProto_SuperDevice>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DynamicPartitionsDeviceInfoProto_SuperDevice& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DynamicPartitionsDeviceInfoProto_SuperDevice& from) { DynamicPartitionsDeviceInfoProto_SuperDevice::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "android.DynamicPartitionsDeviceInfoProto.SuperDevice"; }

  explicit DynamicPartitionsDeviceInfoProto_SuperDevice(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DynamicPartitionsDeviceInfoProto_SuperDevice(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DynamicPartitionsDeviceInfoProto_SuperDevice& from);
  DynamicPartitionsDeviceInfoProto_SuperDevice(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DynamicPartitionsDeviceInfoProto_SuperDevice&& from) noexcept
      : DynamicPartitionsDeviceInfoProto_SuperDevice(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kUsedSizeFieldNumber = 2,
    kTotalSizeFieldNumber = 3,
  };
  // string name = 1;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // uint64 used_size = 2 [json_name = "used_size"];
  void clear_used_size() ;
  [[nodiscard]] ::uint64_t used_size() const;
  void set_used_size(::uint64_t value);

  private:
  ::uint64_t _internal_used_size() const;
  void _internal_set_used_size(::uint64_t value);

  public:
  // uint64 total_size = 3 [json_name = "total_size"];
  void clear_total_size() ;
  [[nodiscard]] ::uint64_t total_size() const;
  void set_total_size(::uint64_t value);

  private:
  ::uint64_t _internal_total_size() const;
  void _internal_set_total_size(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:android.DynamicPartitionsDeviceInfoProto.SuperDevice)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          0, 65,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DynamicPartitionsDeviceInfoProto_SuperDevice& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint64_t used_size_;
    ::uint64_t total_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_dynamic_5fpartitions_5fdevice_5finfo_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED DynamicPartitionsDeviceInfoProto_Partition final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:android.DynamicPartitionsDeviceInfoProto.Partition) */ {
 public:
  inline DynamicPartitionsDeviceInfoProto_Partition() : DynamicPartitionsDeviceInfoProto_Partition(nullptr) {}
  ~DynamicPartitionsDeviceInfoProto_Partition() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DynamicPartitionsDeviceInfoProto_Partition));
  }
#endif

  template <typename = void>
  explicit constexpr DynamicPartitionsDeviceInfoProto_Partition(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline DynamicPartitionsDeviceInfoProto_Partition(const DynamicPartitionsDeviceInfoProto_Partition& from) : DynamicPartitionsDeviceInfoProto_Partition(nullptr, from) {}
  inline DynamicPartitionsDeviceInfoProto_Partition(DynamicPartitionsDeviceInfoProto_Partition&& from) noexcept : DynamicPartitionsDeviceInfoProto_Partition(nullptr, ::std::move(from)) {}
  inline DynamicPartitionsDeviceInfoProto_Partition& operator=(const DynamicPartitionsDeviceInfoProto_Partition& from) {
    CopyFrom(from);
    return *this;
  }
  inline DynamicPartitionsDeviceInfoProto_Partition& operator=(DynamicPartitionsDeviceInfoProto_Partition&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const DynamicPartitionsDeviceInfoProto_Partition& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<DynamicPartitionsDeviceInfoProto_Partition>(&DynamicPartitionsDeviceInfoProto_Partition_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(DynamicPartitionsDeviceInfoProto_Partition& a, DynamicPartitionsDeviceInfoProto_Partition& b) { a.Swap(&b); }
  inline void Swap(DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DynamicPartitionsDeviceInfoProto_Partition>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DynamicPartitionsDeviceInfoProto_Partition& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DynamicPartitionsDeviceInfoProto_Partition& from) { DynamicPartitionsDeviceInfoProto_Partition::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "android.DynamicPartitionsDeviceInfoProto.Partition"; }

  explicit DynamicPartitionsDeviceInfoProto_Partition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DynamicPartitionsDeviceInfoProto_Partition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DynamicPartitionsDeviceInfoProto_Partition& from);
  DynamicPartitionsDeviceInfoProto_Partition(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DynamicPartitionsDeviceInfoProto_Partition&& from) noexcept
      : DynamicPartitionsDeviceInfoProto_Partition(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kGroupNameFieldNumber = 2,
    kFsTypeFieldNumber = 7,
    kSizeFieldNumber = 4,
    kFsSizeFieldNumber = 5,
    kFsUsedFieldNumber = 6,
    kIsDynamicFieldNumber = 3,
  };
  // string name = 1;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // string group_name = 2 [json_name = "group_name"];
  void clear_group_name() ;
  [[nodiscard]] const ::std::string& group_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_group_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_group_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_group_name();
  void set_allocated_group_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_group_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_group_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_group_name();

  public:
  // string fs_type = 7 [json_name = "fs_type"];
  void clear_fs_type() ;
  [[nodiscard]] const ::std::string& fs_type() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fs_type(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fs_type();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fs_type();
  void set_allocated_fs_type(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fs_type() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fs_type(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fs_type();

  public:
  // uint64 size = 4;
  void clear_size() ;
  [[nodiscard]] ::uint64_t size() const;
  void set_size(::uint64_t value);

  private:
  ::uint64_t _internal_size() const;
  void _internal_set_size(::uint64_t value);

  public:
  // uint64 fs_size = 5 [json_name = "fs_size"];
  void clear_fs_size() ;
  [[nodiscard]] ::uint64_t fs_size() const;
  void set_fs_size(::uint64_t value);

  private:
  ::uint64_t _internal_fs_size() const;
  void _internal_set_fs_size(::uint64_t value);

  public:
  // uint64 fs_used = 6 [json_name = "fs_used"];
  void clear_fs_used() ;
  [[nodiscard]] ::uint64_t fs_used() const;
  void set_fs_used(::uint64_t value);

  private:
  ::uint64_t _internal_fs_used() const;
  void _internal_set_fs_used(::uint64_t value);

  public:
  // bool is_dynamic = 3 [json_name = "is_dynamic"];
  void clear_is_dynamic() ;
  [[nodiscard]] bool is_dynamic() const;
  void set_is_dynamic(bool value);

  private:
  bool _internal_is_dynamic() const;
  void _internal_set_is_dynamic(bool value);

  public:
  // @@protoc_insertion_point(class_scope:android.DynamicPartitionsDeviceInfoProto.Partition)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          0, 80,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DynamicPartitionsDeviceInfoProto_Partition& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr group_name_;
    ::google::protobuf::internal::ArenaStringPtr fs_type_;
    ::uint64_t size_;
    ::uint64_t fs_size_;
    ::uint64_t fs_used_;
    bool is_dynamic_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_dynamic_5fpartitions_5fdevice_5finfo_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED DynamicPartitionsDeviceInfoProto_Group final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:android.DynamicPartitionsDeviceInfoProto.Group) */ {
 public:
  inline DynamicPartitionsDeviceInfoProto_Group() : DynamicPartitionsDeviceInfoProto_Group(nullptr) {}
  ~DynamicPartitionsDeviceInfoProto_Group() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DynamicPartitionsDeviceInfoProto_Group));
  }
#endif

  template <typename = void>
  explicit constexpr DynamicPartitionsDeviceInfoProto_Group(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline DynamicPartitionsDeviceInfoProto_Group(const DynamicPartitionsDeviceInfoProto_Group& from) : DynamicPartitionsDeviceInfoProto_Group(nullptr, from) {}
  inline DynamicPartitionsDeviceInfoProto_Group(DynamicPartitionsDeviceInfoProto_Group&& from) noexcept : DynamicPartitionsDeviceInfoProto_Group(nullptr, ::std::move(from)) {}
  inline DynamicPartitionsDeviceInfoProto_Group& operator=(const DynamicPartitionsDeviceInfoProto_Group& from) {
    CopyFrom(from);
    return *this;
  }
  inline DynamicPartitionsDeviceInfoProto_Group& operator=(DynamicPartitionsDeviceInfoProto_Group&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const DynamicPartitionsDeviceInfoProto_Group& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<DynamicPartitionsDeviceInfoProto_Group>(&DynamicPartitionsDeviceInfoProto_Group_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(DynamicPartitionsDeviceInfoProto_Group& a, DynamicPartitionsDeviceInfoProto_Group& b) { a.Swap(&b); }
  inline void Swap(DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DynamicPartitionsDeviceInfoProto_Group>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DynamicPartitionsDeviceInfoProto_Group& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DynamicPartitionsDeviceInfoProto_Group& from) { DynamicPartitionsDeviceInfoProto_Group::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "android.DynamicPartitionsDeviceInfoProto.Group"; }

  explicit DynamicPartitionsDeviceInfoProto_Group(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DynamicPartitionsDeviceInfoProto_Group(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DynamicPartitionsDeviceInfoProto_Group& from);
  DynamicPartitionsDeviceInfoProto_Group(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DynamicPartitionsDeviceInfoProto_Group&& from) noexcept
      : DynamicPartitionsDeviceInfoProto_Group(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kMaximumSizeFieldNumber = 2,
  };
  // string name = 1;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // uint64 maximum_size = 2 [json_name = "maximum_size"];
  void clear_maximum_size() ;
  [[nodiscard]] ::uint64_t maximum_size() const;
  void set_maximum_size(::uint64_t value);

  private:
  ::uint64_t _internal_maximum_size() const;
  void _internal_set_maximum_size(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:android.DynamicPartitionsDeviceInfoProto.Group)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 59,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DynamicPartitionsDeviceInfoProto_Group& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint64_t maximum_size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_dynamic_5fpartitions_5fdevice_5finfo_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED DynamicPartitionsDeviceInfoProto_BlockDevice final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:android.DynamicPartitionsDeviceInfoProto.BlockDevice) */ {
 public:
  inline DynamicPartitionsDeviceInfoProto_BlockDevice() : DynamicPartitionsDeviceInfoProto_BlockDevice(nullptr) {}
  ~DynamicPartitionsDeviceInfoProto_BlockDevice() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DynamicPartitionsDeviceInfoProto_BlockDevice));
  }
#endif

  template <typename = void>
  explicit constexpr DynamicPartitionsDeviceInfoProto_BlockDevice(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline DynamicPartitionsDeviceInfoProto_BlockDevice(const DynamicPartitionsDeviceInfoProto_BlockDevice& from) : DynamicPartitionsDeviceInfoProto_BlockDevice(nullptr, from) {}
  inline DynamicPartitionsDeviceInfoProto_BlockDevice(DynamicPartitionsDeviceInfoProto_BlockDevice&& from) noexcept : DynamicPartitionsDeviceInfoProto_BlockDevice(nullptr, ::std::move(from)) {}
  inline DynamicPartitionsDeviceInfoProto_BlockDevice& operator=(const DynamicPartitionsDeviceInfoProto_BlockDevice& from) {
    CopyFrom(from);
    return *this;
  }
  inline DynamicPartitionsDeviceInfoProto_BlockDevice& operator=(DynamicPartitionsDeviceInfoProto_BlockDevice&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const DynamicPartitionsDeviceInfoProto_BlockDevice& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<DynamicPartitionsDeviceInfoProto_BlockDevice>(&DynamicPartitionsDeviceInfoProto_BlockDevice_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(DynamicPartitionsDeviceInfoProto_BlockDevice& a, DynamicPartitionsDeviceInfoProto_BlockDevice& b) { a.Swap(&b); }
  inline void Swap(DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DynamicPartitionsDeviceInfoProto_BlockDevice>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DynamicPartitionsDeviceInfoProto_BlockDevice& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DynamicPartitionsDeviceInfoProto_BlockDevice& from) { DynamicPartitionsDeviceInfoProto_BlockDevice::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "android.DynamicPartitionsDeviceInfoProto.BlockDevice"; }

  explicit DynamicPartitionsDeviceInfoProto_BlockDevice(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DynamicPartitionsDeviceInfoProto_BlockDevice(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DynamicPartitionsDeviceInfoProto_BlockDevice& from);
  DynamicPartitionsDeviceInfoProto_BlockDevice(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DynamicPartitionsDeviceInfoProto_BlockDevice&& from) noexcept
      : DynamicPartitionsDeviceInfoProto_BlockDevice(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kSizeFieldNumber = 2,
    kBlockSizeFieldNumber = 3,
    kAlignmentFieldNumber = 4,
    kAlignmentOffsetFieldNumber = 5,
  };
  // string name = 1;
  void clear_name() ;
  [[nodiscard]] const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // uint64 size = 2;
  void clear_size() ;
  [[nodiscard]] ::uint64_t size() const;
  void set_size(::uint64_t value);

  private:
  ::uint64_t _internal_size() const;
  void _internal_set_size(::uint64_t value);

  public:
  // uint64 block_size = 3 [json_name = "block_size"];
  void clear_block_size() ;
  [[nodiscard]] ::uint64_t block_size() const;
  void set_block_size(::uint64_t value);

  private:
  ::uint64_t _internal_block_size() const;
  void _internal_set_block_size(::uint64_t value);

  public:
  // uint64 alignment = 4;
  void clear_alignment() ;
  [[nodiscard]] ::uint64_t alignment() const;
  void set_alignment(::uint64_t value);

  private:
  ::uint64_t _internal_alignment() const;
  void _internal_set_alignment(::uint64_t value);

  public:
  // uint64 alignment_offset = 5 [json_name = "alignment_offset"];
  void clear_alignment_offset() ;
  [[nodiscard]] ::uint64_t alignment_offset() const;
  void set_alignment_offset(::uint64_t value);

  private:
  ::uint64_t _internal_alignment_offset() const;
  void _internal_set_alignment_offset(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:android.DynamicPartitionsDeviceInfoProto.BlockDevice)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          0, 65,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DynamicPartitionsDeviceInfoProto_BlockDevice& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint64_t size_;
    ::uint64_t block_size_;
    ::uint64_t alignment_;
    ::uint64_t alignment_offset_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_dynamic_5fpartitions_5fdevice_5finfo_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED DynamicPartitionsDeviceInfoProto final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:android.DynamicPartitionsDeviceInfoProto) */ {
 public:
  inline DynamicPartitionsDeviceInfoProto() : DynamicPartitionsDeviceInfoProto(nullptr) {}
  ~DynamicPartitionsDeviceInfoProto() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DynamicPartitionsDeviceInfoProto* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DynamicPartitionsDeviceInfoProto));
  }
#endif

  template <typename = void>
  explicit constexpr DynamicPartitionsDeviceInfoProto(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline DynamicPartitionsDeviceInfoProto(const DynamicPartitionsDeviceInfoProto& from) : DynamicPartitionsDeviceInfoProto(nullptr, from) {}
  inline DynamicPartitionsDeviceInfoProto(DynamicPartitionsDeviceInfoProto&& from) noexcept : DynamicPartitionsDeviceInfoProto(nullptr, ::std::move(from)) {}
  inline DynamicPartitionsDeviceInfoProto& operator=(const DynamicPartitionsDeviceInfoProto& from) {
    CopyFrom(from);
    return *this;
  }
  inline DynamicPartitionsDeviceInfoProto& operator=(DynamicPartitionsDeviceInfoProto&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const DynamicPartitionsDeviceInfoProto& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<DynamicPartitionsDeviceInfoProto>(&DynamicPartitionsDeviceInfoProto_globals_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(DynamicPartitionsDeviceInfoProto& a, DynamicPartitionsDeviceInfoProto& b) { a.Swap(&b); }
  inline void Swap(DynamicPartitionsDeviceInfoProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DynamicPartitionsDeviceInfoProto* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] DynamicPartitionsDeviceInfoProto* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DynamicPartitionsDeviceInfoProto>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DynamicPartitionsDeviceInfoProto& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DynamicPartitionsDeviceInfoProto& from) { DynamicPartitionsDeviceInfoProto::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DynamicPartitionsDeviceInfoProto* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "android.DynamicPartitionsDeviceInfoProto"; }

  explicit DynamicPartitionsDeviceInfoProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DynamicPartitionsDeviceInfoProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DynamicPartitionsDeviceInfoProto& from);
  DynamicPartitionsDeviceInfoProto(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DynamicPartitionsDeviceInfoProto&& from) noexcept
      : DynamicPartitionsDeviceInfoProto(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using Partition = DynamicPartitionsDeviceInfoProto_Partition;
  using Group = DynamicPartitionsDeviceInfoProto_Group;
  using BlockDevice = DynamicPartitionsDeviceInfoProto_BlockDevice;
  using SuperDevice = DynamicPartitionsDeviceInfoProto_SuperDevice;

  // accessors -------------------------------------------------------
  enum : int {
    kPartitionsFieldNumber = 3,
    kGroupsFieldNumber = 4,
    kBlockDevicesFieldNumber = 5,
    kSuperDeviceFieldNumber = 6,
    kEnabledFieldNumber = 1,
    kRetrofitFieldNumber = 2,
  };
  // repeated .android.DynamicPartitionsDeviceInfoProto.Partition partitions = 3;
  [[nodiscard]] int partitions_size()
      const;
  private:
  int _internal_partitions_size() const;

  public:
  void clear_partitions() ;
  [[nodiscard]] const ::android::DynamicPartitionsDeviceInfoProto_Partition& partitions(int index) const;
  [[nodiscard]] ::android::DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL mutable_partitions(int index);
  ::android::DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL add_partitions();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>&
  partitions() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>* PROTOBUF_NONNULL
  mutable_partitions();

  private:
  const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>& _internal_partitions() const;
  ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>* PROTOBUF_NONNULL _internal_mutable_partitions();

  public:
  // repeated .android.DynamicPartitionsDeviceInfoProto.Group groups = 4;
  [[nodiscard]] int groups_size()
      const;
  private:
  int _internal_groups_size() const;

  public:
  void clear_groups() ;
  [[nodiscard]] const ::android::DynamicPartitionsDeviceInfoProto_Group& groups(int index) const;
  [[nodiscard]] ::android::DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL mutable_groups(int index);
  ::android::DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL add_groups();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>&
  groups() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>* PROTOBUF_NONNULL
  mutable_groups();

  private:
  const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>& _internal_groups() const;
  ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>* PROTOBUF_NONNULL _internal_mutable_groups();

  public:
  // repeated .android.DynamicPartitionsDeviceInfoProto.BlockDevice block_devices = 5 [json_name = "block_devices"];
  [[nodiscard]] int block_devices_size()
      const;
  private:
  int _internal_block_devices_size() const;

  public:
  void clear_block_devices() ;
  [[nodiscard]] const ::android::DynamicPartitionsDeviceInfoProto_BlockDevice& block_devices(int index) const;
  [[nodiscard]] ::android::DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL mutable_block_devices(int index);
  ::android::DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL add_block_devices();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>&
  block_devices() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>* PROTOBUF_NONNULL
  mutable_block_devices();

  private:
  const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>& _internal_block_devices() const;
  ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>* PROTOBUF_NONNULL _internal_mutable_block_devices();

  public:
  // .android.DynamicPartitionsDeviceInfoProto.SuperDevice super_device = 6 [json_name = "super_device"];
  [[nodiscard]] bool has_super_device()
      const;
  void clear_super_device() ;
  [[nodiscard]] const ::android::DynamicPartitionsDeviceInfoProto_SuperDevice& super_device() const;
  [[nodiscard]] ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE release_super_device();
  ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL mutable_super_device();
  void set_allocated_super_device(::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_super_device(::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE value);
  ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE unsafe_arena_release_super_device();

  private:
  const ::android::DynamicPartitionsDeviceInfoProto_SuperDevice& _internal_super_device() const;
  ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL _internal_mutable_super_device();

  public:
  // bool enabled = 1;
  void clear_enabled() ;
  [[nodiscard]] bool enabled() const;
  void set_enabled(bool value);

  private:
  bool _internal_enabled() const;
  void _internal_set_enabled(bool value);

  public:
  // bool retrofit = 2;
  void clear_retrofit() ;
  [[nodiscard]] bool retrofit() const;
  void set_retrofit(bool value);

  private:
  bool _internal_retrofit() const;
  void _internal_set_retrofit(bool value);

  public:
  // @@protoc_insertion_point(class_scope:android.DynamicPartitionsDeviceInfoProto)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          4, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DynamicPartitionsDeviceInfoProto& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::android::DynamicPartitionsDeviceInfoProto_Partition > partitions_;
    ::google::protobuf::RepeatedPtrField< ::android::DynamicPartitionsDeviceInfoProto_Group > groups_;
    ::google::protobuf::RepeatedPtrField< ::android::DynamicPartitionsDeviceInfoProto_BlockDevice > block_devices_;
    ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE super_device_;
    bool enabled_;
    bool retrofit_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_dynamic_5fpartitions_5fdevice_5finfo_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// DynamicPartitionsDeviceInfoProto_Partition

// string name = 1;
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Partition::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DynamicPartitionsDeviceInfoProto_Partition::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.name)
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Partition::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.Partition.name)
  return _s;
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Partition::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Partition::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto_Partition::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.Partition.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.Partition.name)
}

// string group_name = 2 [json_name = "group_name"];
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_group_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Partition::group_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.group_name)
  return _internal_group_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DynamicPartitionsDeviceInfoProto_Partition::set_group_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.group_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.group_name)
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Partition::mutable_group_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_group_name();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.Partition.group_name)
  return _s;
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Partition::_internal_group_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.group_name_.Get();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_group_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Partition::_internal_mutable_group_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.group_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto_Partition::release_group_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.Partition.group_name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.group_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.group_name_.Set("", GetArena());
  }
  return released;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_allocated_group_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.group_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.group_name_.IsDefault()) {
    _impl_.group_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.Partition.group_name)
}

// bool is_dynamic = 3 [json_name = "is_dynamic"];
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_is_dynamic() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.is_dynamic_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline bool DynamicPartitionsDeviceInfoProto_Partition::is_dynamic() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.is_dynamic)
  return _internal_is_dynamic();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_is_dynamic(bool value) {
  _internal_set_is_dynamic(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.is_dynamic)
}
inline bool DynamicPartitionsDeviceInfoProto_Partition::_internal_is_dynamic() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.is_dynamic_;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_is_dynamic(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.is_dynamic_ = value;
}

// uint64 size = 4;
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Partition::size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.size)
  return _internal_size();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_size(::uint64_t value) {
  _internal_set_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Partition::_internal_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.size_;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = value;
}

// uint64 fs_size = 5 [json_name = "fs_size"];
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_fs_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fs_size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Partition::fs_size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.fs_size)
  return _internal_fs_size();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_fs_size(::uint64_t value) {
  _internal_set_fs_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.fs_size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Partition::_internal_fs_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fs_size_;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_fs_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fs_size_ = value;
}

// uint64 fs_used = 6 [json_name = "fs_used"];
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_fs_used() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fs_used_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Partition::fs_used() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.fs_used)
  return _internal_fs_used();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_fs_used(::uint64_t value) {
  _internal_set_fs_used(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.fs_used)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Partition::_internal_fs_used() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fs_used_;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_fs_used(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fs_used_ = value;
}

// string fs_type = 7 [json_name = "fs_type"];
inline void DynamicPartitionsDeviceInfoProto_Partition::clear_fs_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fs_type_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Partition::fs_type() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Partition.fs_type)
  return _internal_fs_type();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DynamicPartitionsDeviceInfoProto_Partition::set_fs_type(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.fs_type_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Partition.fs_type)
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Partition::mutable_fs_type()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_fs_type();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.Partition.fs_type)
  return _s;
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Partition::_internal_fs_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fs_type_.Get();
}
inline void DynamicPartitionsDeviceInfoProto_Partition::_internal_set_fs_type(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fs_type_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Partition::_internal_mutable_fs_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fs_type_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto_Partition::release_fs_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.Partition.fs_type)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.fs_type_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fs_type_.Set("", GetArena());
  }
  return released;
}
inline void DynamicPartitionsDeviceInfoProto_Partition::set_allocated_fs_type(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.fs_type_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fs_type_.IsDefault()) {
    _impl_.fs_type_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.Partition.fs_type)
}

// -------------------------------------------------------------------

// DynamicPartitionsDeviceInfoProto_Group

// string name = 1;
inline void DynamicPartitionsDeviceInfoProto_Group::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Group::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Group.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DynamicPartitionsDeviceInfoProto_Group::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Group.name)
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Group::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.Group.name)
  return _s;
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_Group::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void DynamicPartitionsDeviceInfoProto_Group::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_Group::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto_Group::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.Group.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void DynamicPartitionsDeviceInfoProto_Group::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.Group.name)
}

// uint64 maximum_size = 2 [json_name = "maximum_size"];
inline void DynamicPartitionsDeviceInfoProto_Group::clear_maximum_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.maximum_size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Group::maximum_size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.Group.maximum_size)
  return _internal_maximum_size();
}
inline void DynamicPartitionsDeviceInfoProto_Group::set_maximum_size(::uint64_t value) {
  _internal_set_maximum_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.Group.maximum_size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_Group::_internal_maximum_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.maximum_size_;
}
inline void DynamicPartitionsDeviceInfoProto_Group::_internal_set_maximum_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.maximum_size_ = value;
}

// -------------------------------------------------------------------

// DynamicPartitionsDeviceInfoProto_BlockDevice

// string name = 1;
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_BlockDevice::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.BlockDevice.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DynamicPartitionsDeviceInfoProto_BlockDevice::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.BlockDevice.name)
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_BlockDevice::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.BlockDevice.name)
  return _s;
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto_BlockDevice::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.BlockDevice.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.BlockDevice.name)
}

// uint64 size = 2;
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::clear_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.BlockDevice.size)
  return _internal_size();
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::set_size(::uint64_t value) {
  _internal_set_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.BlockDevice.size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.size_;
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_set_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = value;
}

// uint64 block_size = 3 [json_name = "block_size"];
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::clear_block_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.block_size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::block_size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.BlockDevice.block_size)
  return _internal_block_size();
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::set_block_size(::uint64_t value) {
  _internal_set_block_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.BlockDevice.block_size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_block_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.block_size_;
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_set_block_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.block_size_ = value;
}

// uint64 alignment = 4;
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::clear_alignment() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alignment_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::alignment() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.BlockDevice.alignment)
  return _internal_alignment();
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::set_alignment(::uint64_t value) {
  _internal_set_alignment(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.BlockDevice.alignment)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_alignment() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.alignment_;
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_set_alignment(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alignment_ = value;
}

// uint64 alignment_offset = 5 [json_name = "alignment_offset"];
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::clear_alignment_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alignment_offset_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::alignment_offset() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.BlockDevice.alignment_offset)
  return _internal_alignment_offset();
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::set_alignment_offset(::uint64_t value) {
  _internal_set_alignment_offset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.BlockDevice.alignment_offset)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_alignment_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.alignment_offset_;
}
inline void DynamicPartitionsDeviceInfoProto_BlockDevice::_internal_set_alignment_offset(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.alignment_offset_ = value;
}

// -------------------------------------------------------------------

// DynamicPartitionsDeviceInfoProto_SuperDevice

// string name = 1;
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_SuperDevice::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.SuperDevice.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void DynamicPartitionsDeviceInfoProto_SuperDevice::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.SuperDevice.name)
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_SuperDevice::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.SuperDevice.name)
  return _s;
}
inline const ::std::string& DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto_SuperDevice::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.SuperDevice.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.SuperDevice.name)
}

// uint64 used_size = 2 [json_name = "used_size"];
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::clear_used_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.used_size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_SuperDevice::used_size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.SuperDevice.used_size)
  return _internal_used_size();
}
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::set_used_size(::uint64_t value) {
  _internal_set_used_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.SuperDevice.used_size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_used_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.used_size_;
}
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_set_used_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.used_size_ = value;
}

// uint64 total_size = 3 [json_name = "total_size"];
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::clear_total_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.total_size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_SuperDevice::total_size() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.SuperDevice.total_size)
  return _internal_total_size();
}
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::set_total_size(::uint64_t value) {
  _internal_set_total_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.SuperDevice.total_size)
}
inline ::uint64_t DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_total_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.total_size_;
}
inline void DynamicPartitionsDeviceInfoProto_SuperDevice::_internal_set_total_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.total_size_ = value;
}

// -------------------------------------------------------------------

// DynamicPartitionsDeviceInfoProto

// bool enabled = 1;
inline void DynamicPartitionsDeviceInfoProto::clear_enabled() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.enabled_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline bool DynamicPartitionsDeviceInfoProto::enabled() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.enabled)
  return _internal_enabled();
}
inline void DynamicPartitionsDeviceInfoProto::set_enabled(bool value) {
  _internal_set_enabled(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.enabled)
}
inline bool DynamicPartitionsDeviceInfoProto::_internal_enabled() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.enabled_;
}
inline void DynamicPartitionsDeviceInfoProto::_internal_set_enabled(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.enabled_ = value;
}

// bool retrofit = 2;
inline void DynamicPartitionsDeviceInfoProto::clear_retrofit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.retrofit_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline bool DynamicPartitionsDeviceInfoProto::retrofit() const {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.retrofit)
  return _internal_retrofit();
}
inline void DynamicPartitionsDeviceInfoProto::set_retrofit(bool value) {
  _internal_set_retrofit(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:android.DynamicPartitionsDeviceInfoProto.retrofit)
}
inline bool DynamicPartitionsDeviceInfoProto::_internal_retrofit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.retrofit_;
}
inline void DynamicPartitionsDeviceInfoProto::_internal_set_retrofit(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.retrofit_ = value;
}

// repeated .android.DynamicPartitionsDeviceInfoProto.Partition partitions = 3;
inline int DynamicPartitionsDeviceInfoProto::_internal_partitions_size() const {
  return _internal_partitions().size();
}
inline int DynamicPartitionsDeviceInfoProto::partitions_size() const {
  return _internal_partitions_size();
}
inline void DynamicPartitionsDeviceInfoProto::clear_partitions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.partitions_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::android::DynamicPartitionsDeviceInfoProto_Partition& DynamicPartitionsDeviceInfoProto::partitions(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.partitions)
  return _internal_partitions().Get(index);
}
inline ::android::DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::mutable_partitions(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.partitions)
  return _internal_mutable_partitions()->Mutable(index);
}
inline ::android::DynamicPartitionsDeviceInfoProto_Partition* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::add_partitions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::android::DynamicPartitionsDeviceInfoProto_Partition* _add =
      _internal_mutable_partitions()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:android.DynamicPartitionsDeviceInfoProto.partitions)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>& DynamicPartitionsDeviceInfoProto::partitions() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:android.DynamicPartitionsDeviceInfoProto.partitions)
  return _internal_partitions();
}
inline ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>* PROTOBUF_NONNULL
DynamicPartitionsDeviceInfoProto::mutable_partitions() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:android.DynamicPartitionsDeviceInfoProto.partitions)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_partitions();
}
inline const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>&
DynamicPartitionsDeviceInfoProto::_internal_partitions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.partitions_;
}
inline ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Partition>* PROTOBUF_NONNULL
DynamicPartitionsDeviceInfoProto::_internal_mutable_partitions() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.partitions_;
}

// repeated .android.DynamicPartitionsDeviceInfoProto.Group groups = 4;
inline int DynamicPartitionsDeviceInfoProto::_internal_groups_size() const {
  return _internal_groups().size();
}
inline int DynamicPartitionsDeviceInfoProto::groups_size() const {
  return _internal_groups_size();
}
inline void DynamicPartitionsDeviceInfoProto::clear_groups() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.groups_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::android::DynamicPartitionsDeviceInfoProto_Group& DynamicPartitionsDeviceInfoProto::groups(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.groups)
  return _internal_groups().Get(index);
}
inline ::android::DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::mutable_groups(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.groups)
  return _internal_mutable_groups()->Mutable(index);
}
inline ::android::DynamicPartitionsDeviceInfoProto_Group* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::add_groups()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::android::DynamicPartitionsDeviceInfoProto_Group* _add =
      _internal_mutable_groups()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:android.DynamicPartitionsDeviceInfoProto.groups)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>& DynamicPartitionsDeviceInfoProto::groups() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:android.DynamicPartitionsDeviceInfoProto.groups)
  return _internal_groups();
}
inline ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>* PROTOBUF_NONNULL
DynamicPartitionsDeviceInfoProto::mutable_groups() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:android.DynamicPartitionsDeviceInfoProto.groups)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_groups();
}
inline const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>&
DynamicPartitionsDeviceInfoProto::_internal_groups() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.groups_;
}
inline ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_Group>* PROTOBUF_NONNULL
DynamicPartitionsDeviceInfoProto::_internal_mutable_groups() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.groups_;
}

// repeated .android.DynamicPartitionsDeviceInfoProto.BlockDevice block_devices = 5 [json_name = "block_devices"];
inline int DynamicPartitionsDeviceInfoProto::_internal_block_devices_size() const {
  return _internal_block_devices().size();
}
inline int DynamicPartitionsDeviceInfoProto::block_devices_size() const {
  return _internal_block_devices_size();
}
inline void DynamicPartitionsDeviceInfoProto::clear_block_devices() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.block_devices_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::android::DynamicPartitionsDeviceInfoProto_BlockDevice& DynamicPartitionsDeviceInfoProto::block_devices(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.block_devices)
  return _internal_block_devices().Get(index);
}
inline ::android::DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::mutable_block_devices(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.block_devices)
  return _internal_mutable_block_devices()->Mutable(index);
}
inline ::android::DynamicPartitionsDeviceInfoProto_BlockDevice* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::add_block_devices()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::android::DynamicPartitionsDeviceInfoProto_BlockDevice* _add =
      _internal_mutable_block_devices()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:android.DynamicPartitionsDeviceInfoProto.block_devices)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>& DynamicPartitionsDeviceInfoProto::block_devices() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:android.DynamicPartitionsDeviceInfoProto.block_devices)
  return _internal_block_devices();
}
inline ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>* PROTOBUF_NONNULL
DynamicPartitionsDeviceInfoProto::mutable_block_devices() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:android.DynamicPartitionsDeviceInfoProto.block_devices)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_block_devices();
}
inline const ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>&
DynamicPartitionsDeviceInfoProto::_internal_block_devices() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.block_devices_;
}
inline ::google::protobuf::RepeatedPtrField<::android::DynamicPartitionsDeviceInfoProto_BlockDevice>* PROTOBUF_NONNULL
DynamicPartitionsDeviceInfoProto::_internal_mutable_block_devices() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.block_devices_;
}

// .android.DynamicPartitionsDeviceInfoProto.SuperDevice super_device = 6 [json_name = "super_device"];
inline bool DynamicPartitionsDeviceInfoProto::has_super_device() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.super_device_ != nullptr);
  return value;
}
inline void DynamicPartitionsDeviceInfoProto::clear_super_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.super_device_ != nullptr) _impl_.super_device_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::android::DynamicPartitionsDeviceInfoProto_SuperDevice& DynamicPartitionsDeviceInfoProto::_internal_super_device() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* p = _impl_.super_device_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::android::DynamicPartitionsDeviceInfoProto_SuperDevice>(&::android::DynamicPartitionsDeviceInfoProto_SuperDevice_globals_);
}
inline const ::android::DynamicPartitionsDeviceInfoProto_SuperDevice& DynamicPartitionsDeviceInfoProto::super_device() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:android.DynamicPartitionsDeviceInfoProto.super_device)
  return _internal_super_device();
}
inline void DynamicPartitionsDeviceInfoProto::unsafe_arena_set_allocated_super_device(
    ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.super_device_);
  }
  _impl_.super_device_ = reinterpret_cast<::android::DynamicPartitionsDeviceInfoProto_SuperDevice*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:android.DynamicPartitionsDeviceInfoProto.super_device)
}
inline ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto::release_super_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* released = _impl_.super_device_;
  _impl_.super_device_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE DynamicPartitionsDeviceInfoProto::unsafe_arena_release_super_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:android.DynamicPartitionsDeviceInfoProto.super_device)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* temp = _impl_.super_device_;
  _impl_.super_device_ = nullptr;
  return temp;
}
inline ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::_internal_mutable_super_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.super_device_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::android::DynamicPartitionsDeviceInfoProto_SuperDevice>(GetArena());
    _impl_.super_device_ = reinterpret_cast<::android::DynamicPartitionsDeviceInfoProto_SuperDevice*>(p);
  }
  return _impl_.super_device_;
}
inline ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NONNULL DynamicPartitionsDeviceInfoProto::mutable_super_device()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::android::DynamicPartitionsDeviceInfoProto_SuperDevice* _msg = _internal_mutable_super_device();
  // @@protoc_insertion_point(field_mutable:android.DynamicPartitionsDeviceInfoProto.super_device)
  return _msg;
}
inline void DynamicPartitionsDeviceInfoProto::set_allocated_super_device(::android::DynamicPartitionsDeviceInfoProto_SuperDevice* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.super_device_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.super_device_ = reinterpret_cast<::android::DynamicPartitionsDeviceInfoProto_SuperDevice*>(value);
  // @@protoc_insertion_point(field_set_allocated:android.DynamicPartitionsDeviceInfoProto.super_device)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace android


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // dynamic_5fpartitions_5fdevice_5finfo_2eproto_2epb_2eh
