// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: key_type.proto
// Protobuf C++ Version: 7.35.0

#ifndef key_5ftype_2eproto_2epb_2eh
#define key_5ftype_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_key_5ftype_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_key_5ftype_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_key_5ftype_2eproto;
}  // extern "C"
namespace adb {
namespace proto {
enum KeyType : int;
extern const uint32_t KeyType_internal_data_[];
}  // namespace proto
}  // namespace adb
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::adb::proto::KeyType_internal_data_>
    internal::EnumTraitsImpl::value<::adb::proto::KeyType>;
}  // namespace protobuf
}  // namespace google

namespace adb {
namespace proto {
enum KeyType : int {
  RSA_2048 = 0,
  KeyType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  KeyType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t KeyType_internal_data_[];
inline constexpr KeyType KeyType_MIN =
    static_cast<KeyType>(0);
inline constexpr KeyType KeyType_MAX =
    static_cast<KeyType>(0);
[[nodiscard]] inline bool KeyType_IsValid(int value) {
  return 0 <= value && value <= 0;
}
inline constexpr int KeyType_ARRAYSIZE = 0 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
KeyType_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(KeyType) {
  return KeyType_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& KeyType_Name(T value) {
  static_assert(::std::is_same<T, KeyType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to KeyType_Name().");
  return KeyType_Name(static_cast<KeyType>(value));
}
template <>
[[nodiscard]] inline const ::std::string& KeyType_Name(KeyType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<KeyType_descriptor, 0, 0>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool KeyType_Parse(
    ::absl::string_view name, KeyType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<KeyType>(KeyType_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================



// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace proto
}  // namespace adb


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::adb::proto::KeyType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::adb::proto::KeyType>() {
  return ::adb::proto::KeyType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // key_5ftype_2eproto_2epb_2eh
