// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: app_processes.proto
// Protobuf C++ Version: 7.35.0

#include "app_processes.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::adb::proto::ProcessEntry
        {&::_pbi::kDescriptorMethods, &::descriptor_table_app_5fprocesses_2eproto, /* tracker*/ nullptr,},
        // ::adb::proto::AppProcesses
        {&::_pbi::kDescriptorMethods, &::descriptor_table_app_5fprocesses_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace adb {
namespace proto {
class ProcessEntry::_Internal {
 public:
  using HasBits = decltype(::std::declval<ProcessEntry>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_._has_bits_);
};

constexpr ProcessEntry::ParseTableT_ ProcessEntry::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_._has_bits_),
      0, // no _extensions_
      9, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::adb::proto::ProcessEntry>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // int64 pid = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ProcessEntry, _impl_.pid_), 3>(),
       {8, 3, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.pid_)}},
      // bool debuggable = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ProcessEntry, _impl_.debuggable_), 6>(),
       {16, 6, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.debuggable_)}},
      // bool profileable = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ProcessEntry, _impl_.profileable_), 7>(),
       {24, 7, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.profileable_)}},
      // string architecture = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 1, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.architecture_)}},
      // optional int64 user_id = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ProcessEntry, _impl_.user_id_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.user_id_)}},
      // optional string process_name = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 2, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.process_name_)}},
      // repeated string package_names = 7;
      {::_pbi::TcParser::FastUR1,
       {58, 0, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.package_names_)}},
      // optional bool waiting_for_debugger = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(ProcessEntry, _impl_.waiting_for_debugger_), 8>(),
       {64, 8, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.waiting_for_debugger_)}},
      // optional int64 uid = 9;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ProcessEntry, _impl_.uid_), 5>(),
       {72, 5, 0,
        PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.uid_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 pid = 1;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.pid_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // bool debuggable = 2;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.debuggable_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // bool profileable = 3;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.profileable_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // string architecture = 4;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.architecture_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // optional int64 user_id = 5;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.user_id_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // optional string process_name = 6;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.process_name_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string package_names = 7;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.package_names_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // optional bool waiting_for_debugger = 8;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.waiting_for_debugger_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // optional int64 uid = 9;
      {PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.uid_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
      "\26\0\0\0\14\0\14\15\0\0\0\0\0\0\0\0"
      "adb.proto.ProcessEntry"
      "architecture"
      "process_name"
      "package_names"
    }},
  };
}


inline constexpr ProcessEntry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        package_names_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::adb::proto::ProcessEntry,
            PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.package_names_)>()
         }
        ,
        architecture_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        process_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        pid_{::int64_t{0}},
        user_id_{::int64_t{0}},
        uid_{::int64_t{0}},
        debuggable_{false},
        profileable_{false},
        waiting_for_debugger_{false} {}

template <typename>
constexpr ProcessEntry::ProcessEntry(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ProcessEntry::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ProcessEntry(arena);
}
constexpr auto ProcessEntry::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ProcessEntry), alignof(ProcessEntry));
}
constexpr auto ProcessEntry::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ProcessEntry::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<ProcessEntry>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ProcessEntry::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<ProcessEntry>(), &ProcessEntry::ByteSizeLong,
              &ProcessEntry::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_app_5fprocesses_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ProcessEntryGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ProcessEntryGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ProcessEntry_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ProcessEntry::InternalGenerateClassData_(
            _default, &ProcessEntry_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ProcessEntry>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ProcessEntryGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ProcessEntry _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ProcessEntry>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ProcessEntryGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ProcessEntryGlobalsTypeInternal ProcessEntry_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ProcessEntry_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ProcessEntry_globals_.GetClassData();
#else
  return ProcessEntry_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AppProcesses::_Internal {
 public:
  using HasBits = decltype(::std::declval<AppProcesses>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AppProcesses, _impl_._has_bits_);
};

constexpr AppProcesses::ParseTableT_ AppProcesses::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AppProcesses, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::adb::proto::AppProcesses>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .adb.proto.ProcessEntry process = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AppProcesses, _impl_.process_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .adb.proto.ProcessEntry process = 1;
      {PROTOBUF_FIELD_OFFSET(AppProcesses, _impl_.process_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::adb::proto::ProcessEntry>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::adb::proto::ProcessEntry_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr AppProcesses::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        process_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::adb::proto::AppProcesses,
            PROTOBUF_FIELD_OFFSET(::adb::proto::AppProcesses, _impl_.process_)>()
         }
     {}

template <typename>
constexpr AppProcesses::AppProcesses(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AppProcesses::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AppProcesses(arena);
}
constexpr auto AppProcesses::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AppProcesses), alignof(AppProcesses));
}
constexpr auto AppProcesses::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AppProcesses::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AppProcesses>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AppProcesses::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AppProcesses>(), &AppProcesses::ByteSizeLong,
              &AppProcesses::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AppProcesses, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_app_5fprocesses_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AppProcessesGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AppProcessesGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AppProcesses_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AppProcesses::InternalGenerateClassData_(
            _default, &AppProcesses_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AppProcesses>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AppProcessesGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AppProcesses _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AppProcesses>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AppProcessesGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AppProcessesGlobalsTypeInternal AppProcesses_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AppProcesses_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AppProcesses_globals_.GetClassData();
#else
  return AppProcesses_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace proto
}  // namespace adb
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_app_5fprocesses_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_app_5fprocesses_2eproto = nullptr;
const ::uint32_t
    TableStruct_app_5fprocesses_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.pid_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.debuggable_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.profileable_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.architecture_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.user_id_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.process_name_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.package_names_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.waiting_for_debugger_),
        PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.uid_),
        3,
        6,
        7,
        1,
        4,
        2,
        0,
        8,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::adb::proto::AppProcesses, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::adb::proto::AppProcesses, _impl_.process_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::adb::proto::ProcessEntry)},
        {21, sizeof(::adb::proto::AppProcesses)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::adb::proto::ProcessEntry_globals_,
        &::adb::proto::AppProcesses_globals_,
};
const char descriptor_table_protodef_app_5fprocesses_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\023app_processes.proto\022\tadb.proto\"\225\002\n\014Pro"
    "cessEntry\022\013\n\003pid\030\001 \001(\003\022\022\n\ndebuggable\030\002 \001"
    "(\010\022\023\n\013profileable\030\003 \001(\010\022\024\n\014architecture\030"
    "\004 \001(\t\022\024\n\007user_id\030\005 \001(\003H\000\210\001\001\022\031\n\014process_n"
    "ame\030\006 \001(\tH\001\210\001\001\022\025\n\rpackage_names\030\007 \003(\t\022!\n"
    "\024waiting_for_debugger\030\010 \001(\010H\002\210\001\001\022\020\n\003uid\030"
    "\t \001(\003H\003\210\001\001B\n\n\010_user_idB\017\n\r_process_nameB"
    "\027\n\025_waiting_for_debuggerB\006\n\004_uid\"8\n\014AppP"
    "rocesses\022(\n\007process\030\001 \003(\0132\027.adb.proto.Pr"
    "ocessEntryB2\n\035com.android.server.adb.pro"
    "tosB\021AppProcessesProtob\006proto3"
};
static ::absl::once_flag descriptor_table_app_5fprocesses_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_app_5fprocesses_2eproto = {
    false,
    false,
    430,
    descriptor_table_protodef_app_5fprocesses_2eproto,
    "app_processes.proto",
    &descriptor_table_app_5fprocesses_2eproto_once,
    nullptr,
    0,
    2,
    schemas,
    file_message_globals,
    TableStruct_app_5fprocesses_2eproto::offsets,
    file_level_enum_descriptors_app_5fprocesses_2eproto,
    file_level_service_descriptors_app_5fprocesses_2eproto,
};
namespace adb {
namespace proto {
// ===================================================================

ProcessEntry::ProcessEntry(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ProcessEntry_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:adb.proto.ProcessEntry)
}
PROTOBUF_NDEBUG_INLINE ProcessEntry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::adb::proto::ProcessEntry& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        package_names_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::adb::proto::ProcessEntry,
              PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.package_names_)>()
          , from.package_names_
        }
        ,
        architecture_(arena, from.architecture_),
        process_name_(arena, from.process_name_) {}

ProcessEntry::ProcessEntry(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ProcessEntry& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, ProcessEntry_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ProcessEntry* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, pid_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, pid_),
           offsetof(Impl_, waiting_for_debugger_) -
               offsetof(Impl_, pid_) +
               sizeof(Impl_::waiting_for_debugger_));

  // @@protoc_insertion_point(copy_constructor:adb.proto.ProcessEntry)
}
PROTOBUF_NDEBUG_INLINE ProcessEntry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        package_names_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::adb::proto::ProcessEntry,
            PROTOBUF_FIELD_OFFSET(::adb::proto::ProcessEntry, _impl_.package_names_)>()
         }
        ,
        architecture_(arena),
        process_name_(arena) {}

inline void ProcessEntry::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, pid_),
           0,
           offsetof(Impl_, waiting_for_debugger_) -
               offsetof(Impl_, pid_) +
               sizeof(Impl_::waiting_for_debugger_));
}
ProcessEntry::~ProcessEntry() {
  // @@protoc_insertion_point(destructor:adb.proto.ProcessEntry)
  SharedDtor(*this);
}
inline void ProcessEntry::SharedDtor(MessageLite& self) {
  ProcessEntry& this_ = static_cast<ProcessEntry&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.architecture_.Destroy();
  this_._impl_.process_name_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ProcessEntry_class_data_ =
        ProcessEntry::InternalGenerateClassData_(ProcessEntry_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ProcessEntry::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ProcessEntry_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ProcessEntry_class_data_.tc_table);
  return ProcessEntry_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ProcessEntry::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ProcessEntry_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ProcessEntry_globals_));
  return ProcessEntry_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ProcessEntry::ParseTableT_
    ProcessEntry::_table_ =
        ProcessEntry::InternalGenerateParseTable_(ProcessEntry_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ProcessEntry::Clear() {
// @@protoc_insertion_point(message_clear_start:adb.proto.ProcessEntry)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.package_names_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.architecture_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _impl_.process_name_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000f8U)) {
    ::memset(&_impl_.pid_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.profileable_) -
        reinterpret_cast<char*>(&_impl_.pid_)) + sizeof(_impl_.profileable_));
  }
  _impl_.waiting_for_debugger_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ProcessEntry::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ProcessEntry& this_ = static_cast<const ProcessEntry&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ProcessEntry::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ProcessEntry& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:adb.proto.ProcessEntry)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 pid = 1;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_pid() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_pid(), target);
    }
  }

  // bool debuggable = 2;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_debuggable() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          2, this_._internal_debuggable(), target);
    }
  }

  // bool profileable = 3;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_profileable() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          3, this_._internal_profileable(), target);
    }
  }

  // string architecture = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_architecture().empty()) {
      const ::std::string& _s = this_._internal_architecture();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "adb.proto.ProcessEntry.architecture");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  // optional int64 user_id = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<5>(
            stream, this_._internal_user_id(), target);
  }

  // optional string process_name = 6;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    const ::std::string& _s = this_._internal_process_name();
    ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
        _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "adb.proto.ProcessEntry.process_name");
    target = stream->WriteStringMaybeAliased(6, _s, target);
  }

  // repeated string package_names = 7;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_package_names_size(); i < n; ++i) {
      const auto& s = this_._internal_package_names().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "adb.proto.ProcessEntry.package_names");
      target = stream->WriteString(7, s, target);
    }
  }

  // optional bool waiting_for_debugger = 8;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteBoolToArray(
        8, this_._internal_waiting_for_debugger(), target);
  }

  // optional int64 uid = 9;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<9>(
            stream, this_._internal_uid(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:adb.proto.ProcessEntry)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ProcessEntry::ByteSizeLong(const MessageLite& base) {
  const ProcessEntry& this_ = static_cast<const ProcessEntry&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ProcessEntry::ByteSizeLong() const {
  const ProcessEntry& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:adb.proto.ProcessEntry)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  total_size += static_cast<bool>(0x00000100U & cached_has_bits) * 2;
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated string package_names = 7;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_package_names().size());
      for (int i = 0, n = this_._internal_package_names().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_package_names().Get(i));
      }
    }
    // string architecture = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_architecture().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_architecture());
      }
    }
    // optional string process_name = 6;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_process_name());
    }
    // int64 pid = 1;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_pid() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_pid());
      }
    }
    // optional int64 user_id = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
          this_._internal_user_id());
    }
    // optional int64 uid = 9;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
          this_._internal_uid());
    }
    // bool debuggable = 2;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_debuggable() != 0) {
        total_size += 2;
      }
    }
    // bool profileable = 3;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_profileable() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ProcessEntry::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ProcessEntry*>(&to_msg);
  auto& from = static_cast<const ProcessEntry&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:adb.proto.ProcessEntry)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_package_names()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_package_names());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_architecture().empty()) {
        _this->_internal_set_architecture(from._internal_architecture());
      } else {
        if (_this->_impl_.architecture_.IsDefault()) {
          _this->_internal_set_architecture("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      _this->_internal_set_process_name(from._internal_process_name());
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_pid() != 0) {
        _this->_impl_.pid_ = from._impl_.pid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      _this->_impl_.user_id_ = from._impl_.user_id_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      _this->_impl_.uid_ = from._impl_.uid_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_debuggable() != 0) {
        _this->_impl_.debuggable_ = from._impl_.debuggable_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_profileable() != 0) {
        _this->_impl_.profileable_ = from._impl_.profileable_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    _this->_impl_.waiting_for_debugger_ = from._impl_.waiting_for_debugger_;
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ProcessEntry::CopyFrom(const ProcessEntry& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:adb.proto.ProcessEntry)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ProcessEntry::InternalSwap(ProcessEntry* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.package_names_.InternalSwap(&other->_impl_.package_names_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.architecture_, &other->_impl_.architecture_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.process_name_, &other->_impl_.process_name_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.waiting_for_debugger_)
      + sizeof(ProcessEntry::_impl_.waiting_for_debugger_)
      - PROTOBUF_FIELD_OFFSET(ProcessEntry, _impl_.pid_)>(
          reinterpret_cast<char*>(&_impl_.pid_),
          reinterpret_cast<char*>(&other->_impl_.pid_));
}

::google::protobuf::Metadata ProcessEntry::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AppProcesses::AppProcesses(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppProcesses_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:adb.proto.AppProcesses)
}
PROTOBUF_NDEBUG_INLINE AppProcesses::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::adb::proto::AppProcesses& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        process_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::adb::proto::AppProcesses,
              PROTOBUF_FIELD_OFFSET(::adb::proto::AppProcesses, _impl_.process_)>()
          , from.process_
        }
     {}

AppProcesses::AppProcesses(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AppProcesses& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AppProcesses_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AppProcesses* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:adb.proto.AppProcesses)
}
PROTOBUF_NDEBUG_INLINE AppProcesses::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        process_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::adb::proto::AppProcesses,
            PROTOBUF_FIELD_OFFSET(::adb::proto::AppProcesses, _impl_.process_)>()
         }
     {}

inline void AppProcesses::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AppProcesses::~AppProcesses() {
  // @@protoc_insertion_point(destructor:adb.proto.AppProcesses)
  SharedDtor(*this);
}
inline void AppProcesses::SharedDtor(MessageLite& self) {
  AppProcesses& this_ = static_cast<AppProcesses&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AppProcesses_class_data_ =
        AppProcesses::InternalGenerateClassData_(AppProcesses_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppProcesses::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppProcesses_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AppProcesses_class_data_.tc_table);
  return AppProcesses_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AppProcesses::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AppProcesses_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AppProcesses_globals_));
  return AppProcesses_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AppProcesses::ParseTableT_
    AppProcesses::_table_ =
        AppProcesses::InternalGenerateParseTable_(AppProcesses_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AppProcesses::Clear() {
// @@protoc_insertion_point(message_clear_start:adb.proto.AppProcesses)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.process_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AppProcesses::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AppProcesses& this_ = static_cast<const AppProcesses&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AppProcesses::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AppProcesses& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:adb.proto.AppProcesses)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .adb.proto.ProcessEntry process = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_process_size());
         i < n; i++) {
      const auto& repfield = this_._internal_process().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:adb.proto.AppProcesses)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AppProcesses::ByteSizeLong(const MessageLite& base) {
  const AppProcesses& this_ = static_cast<const AppProcesses&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AppProcesses::ByteSizeLong() const {
  const AppProcesses& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:adb.proto.AppProcesses)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .adb.proto.ProcessEntry process = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_process_size();
      for (const auto& msg : this_._internal_process()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AppProcesses::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AppProcesses*>(&to_msg);
  auto& from = static_cast<const AppProcesses&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:adb.proto.AppProcesses)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_process()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_process());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AppProcesses::CopyFrom(const AppProcesses& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:adb.proto.AppProcesses)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AppProcesses::InternalSwap(AppProcesses* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.process_.InternalSwap(&other->_impl_.process_);
}

::google::protobuf::Metadata AppProcesses::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace proto
}  // namespace adb
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_app_5fprocesses_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
