// SPDX-License-Identifier: Apache-2.0
// Source: https://android.googlesource.com/platform/build/soong/+/refs/heads/master/cc/libbuildversion/include/build/version.h

#ifndef BUILD_VERSION_H
#define BUILD_VERSION_H

#include <string>

namespace android {
	namespace build {
		std::string GetBuildNumber() {
			return "android-tools";
		}
	}
}

#endif
