// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: adb_host.proto
// Protobuf C++ Version: 7.35.0

#ifndef adb_5fhost_2eproto_2epb_2eh
#define adb_5fhost_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7035000
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_adb_5fhost_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_adb_5fhost_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_adb_5fhost_2eproto;
}  // extern "C"
namespace adb {
namespace proto {
enum AdbServerStatus_MdnsBackend : int;
extern const uint32_t AdbServerStatus_MdnsBackend_internal_data_[];
enum AdbServerStatus_UsbBackend : int;
extern const uint32_t AdbServerStatus_UsbBackend_internal_data_[];
enum ConnectionState : int;
extern const uint32_t ConnectionState_internal_data_[];
enum ConnectionType : int;
extern const uint32_t ConnectionType_internal_data_[];
class AdbServerStatus;
struct AdbServerStatusGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern AdbServerStatusGlobalsTypeInternal AdbServerStatus_globals_;
extern const ::google::protobuf::internal::ClassDataFull AdbServerStatus_class_data_;
#else
extern const AdbServerStatusGlobalsTypeInternal AdbServerStatus_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Device;
struct DeviceGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DeviceGlobalsTypeInternal Device_globals_;
extern const ::google::protobuf::internal::ClassDataFull Device_class_data_;
#else
extern const DeviceGlobalsTypeInternal Device_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Devices;
struct DevicesGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DevicesGlobalsTypeInternal Devices_globals_;
extern const ::google::protobuf::internal::ClassDataFull Devices_class_data_;
#else
extern const DevicesGlobalsTypeInternal Devices_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace proto
}  // namespace adb
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::adb::proto::AdbServerStatus_MdnsBackend_internal_data_>
    internal::EnumTraitsImpl::value<::adb::proto::AdbServerStatus_MdnsBackend>;
template <>
internal::EnumTraitsT<::adb::proto::AdbServerStatus_UsbBackend_internal_data_>
    internal::EnumTraitsImpl::value<::adb::proto::AdbServerStatus_UsbBackend>;
template <>
internal::EnumTraitsT<::adb::proto::ConnectionState_internal_data_>
    internal::EnumTraitsImpl::value<::adb::proto::ConnectionState>;
template <>
internal::EnumTraitsT<::adb::proto::ConnectionType_internal_data_>
    internal::EnumTraitsImpl::value<::adb::proto::ConnectionType>;
}  // namespace protobuf
}  // namespace google

namespace adb {
namespace proto {
enum AdbServerStatus_UsbBackend : int {
  AdbServerStatus_UsbBackend_UNKNOWN_USB = 0,
  AdbServerStatus_UsbBackend_NATIVE = 1,
  AdbServerStatus_UsbBackend_LIBUSB = 2,
  AdbServerStatus_UsbBackend_AdbServerStatus_UsbBackend_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  AdbServerStatus_UsbBackend_AdbServerStatus_UsbBackend_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t AdbServerStatus_UsbBackend_internal_data_[];
inline constexpr AdbServerStatus_UsbBackend AdbServerStatus_UsbBackend_UsbBackend_MIN =
    static_cast<AdbServerStatus_UsbBackend>(0);
inline constexpr AdbServerStatus_UsbBackend AdbServerStatus_UsbBackend_UsbBackend_MAX =
    static_cast<AdbServerStatus_UsbBackend>(2);
[[nodiscard]] inline bool AdbServerStatus_UsbBackend_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int AdbServerStatus_UsbBackend_UsbBackend_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
AdbServerStatus_UsbBackend_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(AdbServerStatus_UsbBackend) {
  return AdbServerStatus_UsbBackend_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& AdbServerStatus_UsbBackend_Name(T value) {
  static_assert(::std::is_same<T, AdbServerStatus_UsbBackend>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to UsbBackend_Name().");
  return AdbServerStatus_UsbBackend_Name(static_cast<AdbServerStatus_UsbBackend>(value));
}
template <>
[[nodiscard]] inline const ::std::string& AdbServerStatus_UsbBackend_Name(AdbServerStatus_UsbBackend value) {
  return ::google::protobuf::internal::NameOfDenseEnum<AdbServerStatus_UsbBackend_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool AdbServerStatus_UsbBackend_Parse(
    ::absl::string_view name, AdbServerStatus_UsbBackend* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<AdbServerStatus_UsbBackend>(AdbServerStatus_UsbBackend_descriptor(), name,
                                           value);
}
enum AdbServerStatus_MdnsBackend : int {
  AdbServerStatus_MdnsBackend_UNKNOWN_MDNS = 0,
  AdbServerStatus_MdnsBackend_BONJOUR = 1,
  AdbServerStatus_MdnsBackend_OPENSCREEN = 2,
  AdbServerStatus_MdnsBackend_AdbServerStatus_MdnsBackend_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  AdbServerStatus_MdnsBackend_AdbServerStatus_MdnsBackend_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t AdbServerStatus_MdnsBackend_internal_data_[];
inline constexpr AdbServerStatus_MdnsBackend AdbServerStatus_MdnsBackend_MdnsBackend_MIN =
    static_cast<AdbServerStatus_MdnsBackend>(0);
inline constexpr AdbServerStatus_MdnsBackend AdbServerStatus_MdnsBackend_MdnsBackend_MAX =
    static_cast<AdbServerStatus_MdnsBackend>(2);
[[nodiscard]] inline bool AdbServerStatus_MdnsBackend_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int AdbServerStatus_MdnsBackend_MdnsBackend_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
AdbServerStatus_MdnsBackend_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(AdbServerStatus_MdnsBackend) {
  return AdbServerStatus_MdnsBackend_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& AdbServerStatus_MdnsBackend_Name(T value) {
  static_assert(::std::is_same<T, AdbServerStatus_MdnsBackend>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to MdnsBackend_Name().");
  return AdbServerStatus_MdnsBackend_Name(static_cast<AdbServerStatus_MdnsBackend>(value));
}
template <>
[[nodiscard]] inline const ::std::string& AdbServerStatus_MdnsBackend_Name(AdbServerStatus_MdnsBackend value) {
  return ::google::protobuf::internal::NameOfDenseEnum<AdbServerStatus_MdnsBackend_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool AdbServerStatus_MdnsBackend_Parse(
    ::absl::string_view name, AdbServerStatus_MdnsBackend* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<AdbServerStatus_MdnsBackend>(AdbServerStatus_MdnsBackend_descriptor(), name,
                                           value);
}
enum ConnectionState : int {
  ANY = 0,
  CONNECTING = 1,
  AUTHORIZING = 2,
  UNAUTHORIZED = 3,
  NOPERMISSION = 4,
  DETACHED = 5,
  OFFLINE = 6,
  BOOTLOADER = 7,
  DEVICE = 8,
  HOST = 9,
  RECOVERY = 10,
  SIDELOAD = 11,
  RESCUE = 12,
  ConnectionState_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  ConnectionState_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t ConnectionState_internal_data_[];
inline constexpr ConnectionState ConnectionState_MIN =
    static_cast<ConnectionState>(0);
inline constexpr ConnectionState ConnectionState_MAX =
    static_cast<ConnectionState>(12);
[[nodiscard]] inline bool ConnectionState_IsValid(int value) {
  return 0 <= value && value <= 12;
}
inline constexpr int ConnectionState_ARRAYSIZE = 12 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ConnectionState_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(ConnectionState) {
  return ConnectionState_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& ConnectionState_Name(T value) {
  static_assert(::std::is_same<T, ConnectionState>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to ConnectionState_Name().");
  return ConnectionState_Name(static_cast<ConnectionState>(value));
}
template <>
[[nodiscard]] inline const ::std::string& ConnectionState_Name(ConnectionState value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ConnectionState_descriptor, 0, 12>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool ConnectionState_Parse(
    ::absl::string_view name, ConnectionState* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<ConnectionState>(ConnectionState_descriptor(), name,
                                           value);
}
enum ConnectionType : int {
  UNKNOWN = 0,
  USB = 1,
  SOCKET = 2,
  ConnectionType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  ConnectionType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t ConnectionType_internal_data_[];
inline constexpr ConnectionType ConnectionType_MIN =
    static_cast<ConnectionType>(0);
inline constexpr ConnectionType ConnectionType_MAX =
    static_cast<ConnectionType>(2);
[[nodiscard]] inline bool ConnectionType_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int ConnectionType_ARRAYSIZE = 2 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ConnectionType_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(ConnectionType) {
  return ConnectionType_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& ConnectionType_Name(T value) {
  static_assert(::std::is_same<T, ConnectionType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to ConnectionType_Name().");
  return ConnectionType_Name(static_cast<ConnectionType>(value));
}
template <>
[[nodiscard]] inline const ::std::string& ConnectionType_Name(ConnectionType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ConnectionType_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool ConnectionType_Parse(
    ::absl::string_view name, ConnectionType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<ConnectionType>(ConnectionType_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Device final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:adb.proto.Device) */ {
 public:
  inline Device() : Device(nullptr) {}
  ~Device() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Device* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Device));
  }
#endif

  template <typename = void>
  explicit constexpr Device(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Device(const Device& from) : Device(nullptr, from) {}
  inline Device(Device&& from) noexcept : Device(nullptr, ::std::move(from)) {}
  inline Device& operator=(const Device& from) {
    CopyFrom(from);
    return *this;
  }
  inline Device& operator=(Device&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Device& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Device>(&Device_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Device& a, Device& b) { a.Swap(&b); }
  inline void Swap(Device* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Device* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Device* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Device>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Device& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Device& from) { Device::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Device* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "adb.proto.Device"; }

  explicit Device(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Device(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Device& from);
  Device(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Device&& from) noexcept
      : Device(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSerialFieldNumber = 1,
    kBusAddressFieldNumber = 3,
    kProductFieldNumber = 4,
    kModelFieldNumber = 5,
    kDeviceFieldNumber = 6,
    kStateFieldNumber = 2,
    kConnectionTypeFieldNumber = 7,
    kNegotiatedSpeedFieldNumber = 8,
    kMaxSpeedFieldNumber = 9,
    kTransportIdFieldNumber = 10,
  };
  // string serial = 1;
  void clear_serial() ;
  [[nodiscard]] const ::std::string& serial() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_serial(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_serial();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_serial();
  void set_allocated_serial(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_serial() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_serial(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_serial();

  public:
  // string bus_address = 3;
  void clear_bus_address() ;
  [[nodiscard]] const ::std::string& bus_address() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_bus_address(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_bus_address();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_bus_address();
  void set_allocated_bus_address(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_bus_address() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_bus_address(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_bus_address();

  public:
  // string product = 4;
  void clear_product() ;
  [[nodiscard]] const ::std::string& product() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_product(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_product();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_product();
  void set_allocated_product(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_product() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_product(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_product();

  public:
  // string model = 5;
  void clear_model() ;
  [[nodiscard]] const ::std::string& model() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_model(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_model();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_model();
  void set_allocated_model(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_model() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_model(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_model();

  public:
  // string device = 6;
  void clear_device() ;
  [[nodiscard]] const ::std::string& device() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_device(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_device();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_device();
  void set_allocated_device(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_device() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_device(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_device();

  public:
  // .adb.proto.ConnectionState state = 2;
  void clear_state() ;
  [[nodiscard]] ::adb::proto::ConnectionState state() const;
  void set_state(::adb::proto::ConnectionState value);

  private:
  ::adb::proto::ConnectionState _internal_state() const;
  void _internal_set_state(::adb::proto::ConnectionState value);

  public:
  // .adb.proto.ConnectionType connection_type = 7;
  void clear_connection_type() ;
  [[nodiscard]] ::adb::proto::ConnectionType connection_type() const;
  void set_connection_type(::adb::proto::ConnectionType value);

  private:
  ::adb::proto::ConnectionType _internal_connection_type() const;
  void _internal_set_connection_type(::adb::proto::ConnectionType value);

  public:
  // int64 negotiated_speed = 8;
  void clear_negotiated_speed() ;
  [[nodiscard]] ::int64_t negotiated_speed() const;
  void set_negotiated_speed(::int64_t value);

  private:
  ::int64_t _internal_negotiated_speed() const;
  void _internal_set_negotiated_speed(::int64_t value);

  public:
  // int64 max_speed = 9;
  void clear_max_speed() ;
  [[nodiscard]] ::int64_t max_speed() const;
  void set_max_speed(::int64_t value);

  private:
  ::int64_t _internal_max_speed() const;
  void _internal_set_max_speed(::int64_t value);

  public:
  // int64 transport_id = 10;
  void clear_transport_id() ;
  [[nodiscard]] ::int64_t transport_id() const;
  void set_transport_id(::int64_t value);

  private:
  ::int64_t _internal_transport_id() const;
  void _internal_set_transport_id(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:adb.proto.Device)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 10,
                          0, 68,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Device& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr serial_;
    ::google::protobuf::internal::ArenaStringPtr bus_address_;
    ::google::protobuf::internal::ArenaStringPtr product_;
    ::google::protobuf::internal::ArenaStringPtr model_;
    ::google::protobuf::internal::ArenaStringPtr device_;
    int state_;
    int connection_type_;
    ::int64_t negotiated_speed_;
    ::int64_t max_speed_;
    ::int64_t transport_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_adb_5fhost_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED AdbServerStatus final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:adb.proto.AdbServerStatus) */ {
 public:
  inline AdbServerStatus() : AdbServerStatus(nullptr) {}
  ~AdbServerStatus() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AdbServerStatus* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AdbServerStatus));
  }
#endif

  template <typename = void>
  explicit constexpr AdbServerStatus(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline AdbServerStatus(const AdbServerStatus& from) : AdbServerStatus(nullptr, from) {}
  inline AdbServerStatus(AdbServerStatus&& from) noexcept : AdbServerStatus(nullptr, ::std::move(from)) {}
  inline AdbServerStatus& operator=(const AdbServerStatus& from) {
    CopyFrom(from);
    return *this;
  }
  inline AdbServerStatus& operator=(AdbServerStatus&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const AdbServerStatus& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<AdbServerStatus>(&AdbServerStatus_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(AdbServerStatus& a, AdbServerStatus& b) { a.Swap(&b); }
  inline void Swap(AdbServerStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AdbServerStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] AdbServerStatus* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AdbServerStatus>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AdbServerStatus& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AdbServerStatus& from) { AdbServerStatus::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AdbServerStatus* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "adb.proto.AdbServerStatus"; }

  explicit AdbServerStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AdbServerStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AdbServerStatus& from);
  AdbServerStatus(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AdbServerStatus&& from) noexcept
      : AdbServerStatus(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using UsbBackend = AdbServerStatus_UsbBackend;
  static constexpr UsbBackend UNKNOWN_USB = AdbServerStatus_UsbBackend_UNKNOWN_USB;
  static constexpr UsbBackend NATIVE = AdbServerStatus_UsbBackend_NATIVE;
  static constexpr UsbBackend LIBUSB = AdbServerStatus_UsbBackend_LIBUSB;
  [[nodiscard]] static inline bool UsbBackend_IsValid(int value) {
    return AdbServerStatus_UsbBackend_IsValid(value);
  }
  static constexpr UsbBackend UsbBackend_MIN = AdbServerStatus_UsbBackend_UsbBackend_MIN;
  static constexpr UsbBackend UsbBackend_MAX = AdbServerStatus_UsbBackend_UsbBackend_MAX;
  static constexpr int UsbBackend_ARRAYSIZE = AdbServerStatus_UsbBackend_UsbBackend_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL UsbBackend_descriptor() {
    return AdbServerStatus_UsbBackend_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& UsbBackend_Name(T value) {
    return AdbServerStatus_UsbBackend_Name(value);
  }
  [[nodiscard]] static inline bool UsbBackend_Parse(
      ::absl::string_view name, UsbBackend* PROTOBUF_NONNULL value) {
    return AdbServerStatus_UsbBackend_Parse(name, value);
  }
  using MdnsBackend = AdbServerStatus_MdnsBackend;
  static constexpr MdnsBackend UNKNOWN_MDNS = AdbServerStatus_MdnsBackend_UNKNOWN_MDNS;
  static constexpr MdnsBackend BONJOUR = AdbServerStatus_MdnsBackend_BONJOUR;
  static constexpr MdnsBackend OPENSCREEN = AdbServerStatus_MdnsBackend_OPENSCREEN;
  [[nodiscard]] static inline bool MdnsBackend_IsValid(int value) {
    return AdbServerStatus_MdnsBackend_IsValid(value);
  }
  static constexpr MdnsBackend MdnsBackend_MIN = AdbServerStatus_MdnsBackend_MdnsBackend_MIN;
  static constexpr MdnsBackend MdnsBackend_MAX = AdbServerStatus_MdnsBackend_MdnsBackend_MAX;
  static constexpr int MdnsBackend_ARRAYSIZE = AdbServerStatus_MdnsBackend_MdnsBackend_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL MdnsBackend_descriptor() {
    return AdbServerStatus_MdnsBackend_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& MdnsBackend_Name(T value) {
    return AdbServerStatus_MdnsBackend_Name(value);
  }
  [[nodiscard]] static inline bool MdnsBackend_Parse(
      ::absl::string_view name, MdnsBackend* PROTOBUF_NONNULL value) {
    return AdbServerStatus_MdnsBackend_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kVersionFieldNumber = 5,
    kBuildFieldNumber = 6,
    kExecutableAbsolutePathFieldNumber = 7,
    kLogAbsolutePathFieldNumber = 8,
    kOsFieldNumber = 9,
    kUsbBackendFieldNumber = 1,
    kMdnsBackendFieldNumber = 3,
    kUsbBackendForcedFieldNumber = 2,
    kMdnsBackendForcedFieldNumber = 4,
  };
  // string version = 5;
  void clear_version() ;
  [[nodiscard]] const ::std::string& version() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_version(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_version();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_version();
  void set_allocated_version(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_version() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_version(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_version();

  public:
  // string build = 6;
  void clear_build() ;
  [[nodiscard]] const ::std::string& build() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_build(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_build();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_build();
  void set_allocated_build(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_build() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_build(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_build();

  public:
  // string executable_absolute_path = 7;
  void clear_executable_absolute_path() ;
  [[nodiscard]] const ::std::string& executable_absolute_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_executable_absolute_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_executable_absolute_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_executable_absolute_path();
  void set_allocated_executable_absolute_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_executable_absolute_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_executable_absolute_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_executable_absolute_path();

  public:
  // string log_absolute_path = 8;
  void clear_log_absolute_path() ;
  [[nodiscard]] const ::std::string& log_absolute_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_log_absolute_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_log_absolute_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_log_absolute_path();
  void set_allocated_log_absolute_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_log_absolute_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_log_absolute_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_log_absolute_path();

  public:
  // string os = 9;
  void clear_os() ;
  [[nodiscard]] const ::std::string& os() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_os(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_os();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_os();
  void set_allocated_os(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_os() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_os(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_os();

  public:
  // .adb.proto.AdbServerStatus.UsbBackend usb_backend = 1;
  void clear_usb_backend() ;
  [[nodiscard]] ::adb::proto::AdbServerStatus_UsbBackend usb_backend() const;
  void set_usb_backend(::adb::proto::AdbServerStatus_UsbBackend value);

  private:
  ::adb::proto::AdbServerStatus_UsbBackend _internal_usb_backend() const;
  void _internal_set_usb_backend(::adb::proto::AdbServerStatus_UsbBackend value);

  public:
  // .adb.proto.AdbServerStatus.MdnsBackend mdns_backend = 3;
  void clear_mdns_backend() ;
  [[nodiscard]] ::adb::proto::AdbServerStatus_MdnsBackend mdns_backend() const;
  void set_mdns_backend(::adb::proto::AdbServerStatus_MdnsBackend value);

  private:
  ::adb::proto::AdbServerStatus_MdnsBackend _internal_mdns_backend() const;
  void _internal_set_mdns_backend(::adb::proto::AdbServerStatus_MdnsBackend value);

  public:
  // bool usb_backend_forced = 2;
  void clear_usb_backend_forced() ;
  [[nodiscard]] bool usb_backend_forced() const;
  void set_usb_backend_forced(bool value);

  private:
  bool _internal_usb_backend_forced() const;
  void _internal_set_usb_backend_forced(bool value);

  public:
  // bool mdns_backend_forced = 4;
  void clear_mdns_backend_forced() ;
  [[nodiscard]] bool mdns_backend_forced() const;
  void set_mdns_backend_forced(bool value);

  private:
  bool _internal_mdns_backend_forced() const;
  void _internal_set_mdns_backend_forced(bool value);

  public:
  // @@protoc_insertion_point(class_scope:adb.proto.AdbServerStatus)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          0, 97,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AdbServerStatus& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr version_;
    ::google::protobuf::internal::ArenaStringPtr build_;
    ::google::protobuf::internal::ArenaStringPtr executable_absolute_path_;
    ::google::protobuf::internal::ArenaStringPtr log_absolute_path_;
    ::google::protobuf::internal::ArenaStringPtr os_;
    int usb_backend_;
    int mdns_backend_;
    bool usb_backend_forced_;
    bool mdns_backend_forced_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_adb_5fhost_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Devices final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:adb.proto.Devices) */ {
 public:
  inline Devices() : Devices(nullptr) {}
  ~Devices() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Devices* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Devices));
  }
#endif

  template <typename = void>
  explicit constexpr Devices(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Devices(const Devices& from) : Devices(nullptr, from) {}
  inline Devices(Devices&& from) noexcept : Devices(nullptr, ::std::move(from)) {}
  inline Devices& operator=(const Devices& from) {
    CopyFrom(from);
    return *this;
  }
  inline Devices& operator=(Devices&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Devices& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Devices>(&Devices_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(Devices& a, Devices& b) { a.Swap(&b); }
  inline void Swap(Devices* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Devices* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Devices* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Devices>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Devices& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Devices& from) { Devices::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Devices* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "adb.proto.Devices"; }

  explicit Devices(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Devices(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Devices& from);
  Devices(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Devices&& from) noexcept
      : Devices(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kDeviceFieldNumber = 1,
  };
  // repeated .adb.proto.Device device = 1;
  [[nodiscard]] int device_size()
      const;
  private:
  int _internal_device_size() const;

  public:
  void clear_device() ;
  [[nodiscard]] const ::adb::proto::Device& device(int index) const;
  [[nodiscard]] ::adb::proto::Device* PROTOBUF_NONNULL mutable_device(int index);
  ::adb::proto::Device* PROTOBUF_NONNULL add_device();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::adb::proto::Device>&
  device() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::adb::proto::Device>* PROTOBUF_NONNULL
  mutable_device();

  private:
  const ::google::protobuf::RepeatedPtrField<::adb::proto::Device>& _internal_device() const;
  ::google::protobuf::RepeatedPtrField<::adb::proto::Device>* PROTOBUF_NONNULL _internal_mutable_device();

  public:
  // @@protoc_insertion_point(class_scope:adb.proto.Devices)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Devices& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::adb::proto::Device > device_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_adb_5fhost_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Device

// string serial = 1;
inline void Device::clear_serial() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serial_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Device::serial() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.Device.serial)
  return _internal_serial();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_serial(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.serial_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.Device.serial)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_serial()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_serial();
  // @@protoc_insertion_point(field_mutable:adb.proto.Device.serial)
  return _s;
}
inline const ::std::string& Device::_internal_serial() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.serial_.Get();
}
inline void Device::_internal_set_serial(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.serial_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_serial() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.serial_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_serial() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.Device.serial)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.serial_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.serial_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_serial(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.serial_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.serial_.IsDefault()) {
    _impl_.serial_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.Device.serial)
}

// .adb.proto.ConnectionState state = 2;
inline void Device::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::adb::proto::ConnectionState Device::state() const {
  // @@protoc_insertion_point(field_get:adb.proto.Device.state)
  return _internal_state();
}
inline void Device::set_state(::adb::proto::ConnectionState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:adb.proto.Device.state)
}
inline ::adb::proto::ConnectionState Device::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::adb::proto::ConnectionState>(_impl_.state_);
}
inline void Device::_internal_set_state(::adb::proto::ConnectionState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// string bus_address = 3;
inline void Device::clear_bus_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bus_address_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Device::bus_address() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.Device.bus_address)
  return _internal_bus_address();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_bus_address(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.bus_address_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.Device.bus_address)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_bus_address()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_bus_address();
  // @@protoc_insertion_point(field_mutable:adb.proto.Device.bus_address)
  return _s;
}
inline const ::std::string& Device::_internal_bus_address() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.bus_address_.Get();
}
inline void Device::_internal_set_bus_address(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.bus_address_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_bus_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.bus_address_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_bus_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.Device.bus_address)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.bus_address_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.bus_address_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_bus_address(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.bus_address_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.bus_address_.IsDefault()) {
    _impl_.bus_address_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.Device.bus_address)
}

// string product = 4;
inline void Device::clear_product() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.product_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& Device::product() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.Device.product)
  return _internal_product();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_product(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.product_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.Device.product)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_product()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_product();
  // @@protoc_insertion_point(field_mutable:adb.proto.Device.product)
  return _s;
}
inline const ::std::string& Device::_internal_product() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.product_.Get();
}
inline void Device::_internal_set_product(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.product_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_product() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.product_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_product() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.Device.product)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.product_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.product_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_product(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.product_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.product_.IsDefault()) {
    _impl_.product_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.Device.product)
}

// string model = 5;
inline void Device::clear_model() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.model_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& Device::model() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.Device.model)
  return _internal_model();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_model(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.model_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.Device.model)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_model()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_model();
  // @@protoc_insertion_point(field_mutable:adb.proto.Device.model)
  return _s;
}
inline const ::std::string& Device::_internal_model() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.model_.Get();
}
inline void Device::_internal_set_model(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.model_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_model() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.model_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_model() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.Device.model)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.model_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.model_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_model(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.model_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.model_.IsDefault()) {
    _impl_.model_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.Device.model)
}

// string device = 6;
inline void Device::clear_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.device_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline const ::std::string& Device::device() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.Device.device)
  return _internal_device();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_device(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.device_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.Device.device)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_device()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_device();
  // @@protoc_insertion_point(field_mutable:adb.proto.Device.device)
  return _s;
}
inline const ::std::string& Device::_internal_device() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.device_.Get();
}
inline void Device::_internal_set_device(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.device_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.device_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.Device.device)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.device_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.device_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_device(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.device_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.device_.IsDefault()) {
    _impl_.device_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.Device.device)
}

// .adb.proto.ConnectionType connection_type = 7;
inline void Device::clear_connection_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.connection_type_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::adb::proto::ConnectionType Device::connection_type() const {
  // @@protoc_insertion_point(field_get:adb.proto.Device.connection_type)
  return _internal_connection_type();
}
inline void Device::set_connection_type(::adb::proto::ConnectionType value) {
  _internal_set_connection_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:adb.proto.Device.connection_type)
}
inline ::adb::proto::ConnectionType Device::_internal_connection_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::adb::proto::ConnectionType>(_impl_.connection_type_);
}
inline void Device::_internal_set_connection_type(::adb::proto::ConnectionType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.connection_type_ = value;
}

// int64 negotiated_speed = 8;
inline void Device::clear_negotiated_speed() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.negotiated_speed_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int64_t Device::negotiated_speed() const {
  // @@protoc_insertion_point(field_get:adb.proto.Device.negotiated_speed)
  return _internal_negotiated_speed();
}
inline void Device::set_negotiated_speed(::int64_t value) {
  _internal_set_negotiated_speed(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:adb.proto.Device.negotiated_speed)
}
inline ::int64_t Device::_internal_negotiated_speed() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.negotiated_speed_;
}
inline void Device::_internal_set_negotiated_speed(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.negotiated_speed_ = value;
}

// int64 max_speed = 9;
inline void Device::clear_max_speed() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_speed_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::int64_t Device::max_speed() const {
  // @@protoc_insertion_point(field_get:adb.proto.Device.max_speed)
  return _internal_max_speed();
}
inline void Device::set_max_speed(::int64_t value) {
  _internal_set_max_speed(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:adb.proto.Device.max_speed)
}
inline ::int64_t Device::_internal_max_speed() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_speed_;
}
inline void Device::_internal_set_max_speed(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_speed_ = value;
}

// int64 transport_id = 10;
inline void Device::clear_transport_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.transport_id_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
}
inline ::int64_t Device::transport_id() const {
  // @@protoc_insertion_point(field_get:adb.proto.Device.transport_id)
  return _internal_transport_id();
}
inline void Device::set_transport_id(::int64_t value) {
  _internal_set_transport_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:adb.proto.Device.transport_id)
}
inline ::int64_t Device::_internal_transport_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.transport_id_;
}
inline void Device::_internal_set_transport_id(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.transport_id_ = value;
}

// -------------------------------------------------------------------

// Devices

// repeated .adb.proto.Device device = 1;
inline int Devices::_internal_device_size() const {
  return _internal_device().size();
}
inline int Devices::device_size() const {
  return _internal_device_size();
}
inline void Devices::clear_device() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.device_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::adb::proto::Device& Devices::device(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.Devices.device)
  return _internal_device().Get(index);
}
inline ::adb::proto::Device* PROTOBUF_NONNULL Devices::mutable_device(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:adb.proto.Devices.device)
  return _internal_mutable_device()->Mutable(index);
}
inline ::adb::proto::Device* PROTOBUF_NONNULL Devices::add_device()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::adb::proto::Device* _add =
      _internal_mutable_device()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:adb.proto.Devices.device)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::adb::proto::Device>& Devices::device() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:adb.proto.Devices.device)
  return _internal_device();
}
inline ::google::protobuf::RepeatedPtrField<::adb::proto::Device>* PROTOBUF_NONNULL
Devices::mutable_device() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:adb.proto.Devices.device)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_device();
}
inline const ::google::protobuf::RepeatedPtrField<::adb::proto::Device>&
Devices::_internal_device() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.device_;
}
inline ::google::protobuf::RepeatedPtrField<::adb::proto::Device>* PROTOBUF_NONNULL
Devices::_internal_mutable_device() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.device_;
}

// -------------------------------------------------------------------

// AdbServerStatus

// .adb.proto.AdbServerStatus.UsbBackend usb_backend = 1;
inline void AdbServerStatus::clear_usb_backend() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.usb_backend_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::adb::proto::AdbServerStatus_UsbBackend AdbServerStatus::usb_backend() const {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.usb_backend)
  return _internal_usb_backend();
}
inline void AdbServerStatus::set_usb_backend(::adb::proto::AdbServerStatus_UsbBackend value) {
  _internal_set_usb_backend(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.usb_backend)
}
inline ::adb::proto::AdbServerStatus_UsbBackend AdbServerStatus::_internal_usb_backend() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::adb::proto::AdbServerStatus_UsbBackend>(_impl_.usb_backend_);
}
inline void AdbServerStatus::_internal_set_usb_backend(::adb::proto::AdbServerStatus_UsbBackend value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.usb_backend_ = value;
}

// bool usb_backend_forced = 2;
inline void AdbServerStatus::clear_usb_backend_forced() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.usb_backend_forced_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline bool AdbServerStatus::usb_backend_forced() const {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.usb_backend_forced)
  return _internal_usb_backend_forced();
}
inline void AdbServerStatus::set_usb_backend_forced(bool value) {
  _internal_set_usb_backend_forced(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.usb_backend_forced)
}
inline bool AdbServerStatus::_internal_usb_backend_forced() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.usb_backend_forced_;
}
inline void AdbServerStatus::_internal_set_usb_backend_forced(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.usb_backend_forced_ = value;
}

// .adb.proto.AdbServerStatus.MdnsBackend mdns_backend = 3;
inline void AdbServerStatus::clear_mdns_backend() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mdns_backend_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::adb::proto::AdbServerStatus_MdnsBackend AdbServerStatus::mdns_backend() const {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.mdns_backend)
  return _internal_mdns_backend();
}
inline void AdbServerStatus::set_mdns_backend(::adb::proto::AdbServerStatus_MdnsBackend value) {
  _internal_set_mdns_backend(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.mdns_backend)
}
inline ::adb::proto::AdbServerStatus_MdnsBackend AdbServerStatus::_internal_mdns_backend() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::adb::proto::AdbServerStatus_MdnsBackend>(_impl_.mdns_backend_);
}
inline void AdbServerStatus::_internal_set_mdns_backend(::adb::proto::AdbServerStatus_MdnsBackend value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mdns_backend_ = value;
}

// bool mdns_backend_forced = 4;
inline void AdbServerStatus::clear_mdns_backend_forced() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mdns_backend_forced_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline bool AdbServerStatus::mdns_backend_forced() const {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.mdns_backend_forced)
  return _internal_mdns_backend_forced();
}
inline void AdbServerStatus::set_mdns_backend_forced(bool value) {
  _internal_set_mdns_backend_forced(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.mdns_backend_forced)
}
inline bool AdbServerStatus::_internal_mdns_backend_forced() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mdns_backend_forced_;
}
inline void AdbServerStatus::_internal_set_mdns_backend_forced(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mdns_backend_forced_ = value;
}

// string version = 5;
inline void AdbServerStatus::clear_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& AdbServerStatus::version() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.version)
  return _internal_version();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AdbServerStatus::set_version(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.version_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.version)
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::mutable_version()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_version();
  // @@protoc_insertion_point(field_mutable:adb.proto.AdbServerStatus.version)
  return _s;
}
inline const ::std::string& AdbServerStatus::_internal_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_.Get();
}
inline void AdbServerStatus::_internal_set_version(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::_internal_mutable_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.version_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AdbServerStatus::release_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.AdbServerStatus.version)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.version_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.version_.Set("", GetArena());
  }
  return released;
}
inline void AdbServerStatus::set_allocated_version(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.version_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.version_.IsDefault()) {
    _impl_.version_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.AdbServerStatus.version)
}

// string build = 6;
inline void AdbServerStatus::clear_build() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.build_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& AdbServerStatus::build() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.build)
  return _internal_build();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AdbServerStatus::set_build(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.build_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.build)
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::mutable_build()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_build();
  // @@protoc_insertion_point(field_mutable:adb.proto.AdbServerStatus.build)
  return _s;
}
inline const ::std::string& AdbServerStatus::_internal_build() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.build_.Get();
}
inline void AdbServerStatus::_internal_set_build(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.build_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::_internal_mutable_build() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.build_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AdbServerStatus::release_build() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.AdbServerStatus.build)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.build_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.build_.Set("", GetArena());
  }
  return released;
}
inline void AdbServerStatus::set_allocated_build(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.build_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.build_.IsDefault()) {
    _impl_.build_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.AdbServerStatus.build)
}

// string executable_absolute_path = 7;
inline void AdbServerStatus::clear_executable_absolute_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.executable_absolute_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& AdbServerStatus::executable_absolute_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.executable_absolute_path)
  return _internal_executable_absolute_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AdbServerStatus::set_executable_absolute_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.executable_absolute_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.executable_absolute_path)
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::mutable_executable_absolute_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_executable_absolute_path();
  // @@protoc_insertion_point(field_mutable:adb.proto.AdbServerStatus.executable_absolute_path)
  return _s;
}
inline const ::std::string& AdbServerStatus::_internal_executable_absolute_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.executable_absolute_path_.Get();
}
inline void AdbServerStatus::_internal_set_executable_absolute_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.executable_absolute_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::_internal_mutable_executable_absolute_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.executable_absolute_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AdbServerStatus::release_executable_absolute_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.AdbServerStatus.executable_absolute_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.executable_absolute_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.executable_absolute_path_.Set("", GetArena());
  }
  return released;
}
inline void AdbServerStatus::set_allocated_executable_absolute_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.executable_absolute_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.executable_absolute_path_.IsDefault()) {
    _impl_.executable_absolute_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.AdbServerStatus.executable_absolute_path)
}

// string log_absolute_path = 8;
inline void AdbServerStatus::clear_log_absolute_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_absolute_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& AdbServerStatus::log_absolute_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.log_absolute_path)
  return _internal_log_absolute_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AdbServerStatus::set_log_absolute_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.log_absolute_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.log_absolute_path)
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::mutable_log_absolute_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_log_absolute_path();
  // @@protoc_insertion_point(field_mutable:adb.proto.AdbServerStatus.log_absolute_path)
  return _s;
}
inline const ::std::string& AdbServerStatus::_internal_log_absolute_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.log_absolute_path_.Get();
}
inline void AdbServerStatus::_internal_set_log_absolute_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_absolute_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::_internal_mutable_log_absolute_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.log_absolute_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AdbServerStatus::release_log_absolute_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.AdbServerStatus.log_absolute_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.log_absolute_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.log_absolute_path_.Set("", GetArena());
  }
  return released;
}
inline void AdbServerStatus::set_allocated_log_absolute_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.log_absolute_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.log_absolute_path_.IsDefault()) {
    _impl_.log_absolute_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.AdbServerStatus.log_absolute_path)
}

// string os = 9;
inline void AdbServerStatus::clear_os() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.os_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline const ::std::string& AdbServerStatus::os() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:adb.proto.AdbServerStatus.os)
  return _internal_os();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AdbServerStatus::set_os(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.os_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:adb.proto.AdbServerStatus.os)
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::mutable_os()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_os();
  // @@protoc_insertion_point(field_mutable:adb.proto.AdbServerStatus.os)
  return _s;
}
inline const ::std::string& AdbServerStatus::_internal_os() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.os_.Get();
}
inline void AdbServerStatus::_internal_set_os(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.os_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AdbServerStatus::_internal_mutable_os() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.os_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AdbServerStatus::release_os() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:adb.proto.AdbServerStatus.os)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.os_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.os_.Set("", GetArena());
  }
  return released;
}
inline void AdbServerStatus::set_allocated_os(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.os_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.os_.IsDefault()) {
    _impl_.os_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:adb.proto.AdbServerStatus.os)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace proto
}  // namespace adb


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::adb::proto::AdbServerStatus_UsbBackend> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::adb::proto::AdbServerStatus_UsbBackend>() {
  return ::adb::proto::AdbServerStatus_UsbBackend_descriptor();
}
template <>
struct is_proto_enum<::adb::proto::AdbServerStatus_MdnsBackend> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::adb::proto::AdbServerStatus_MdnsBackend>() {
  return ::adb::proto::AdbServerStatus_MdnsBackend_descriptor();
}
template <>
struct is_proto_enum<::adb::proto::ConnectionState> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::adb::proto::ConnectionState>() {
  return ::adb::proto::ConnectionState_descriptor();
}
template <>
struct is_proto_enum<::adb::proto::ConnectionType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::adb::proto::ConnectionType>() {
  return ::adb::proto::ConnectionType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // adb_5fhost_2eproto_2epb_2eh
