// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: adb_known_hosts.proto
// Protobuf C++ Version: 7.35.0

#include "adb_known_hosts.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::adb::proto::HostInfo
        {&::_pbi::kDescriptorMethods, &::descriptor_table_adb_5fknown_5fhosts_2eproto, /* tracker*/ nullptr,},
        // ::adb::proto::AdbKnownHosts
        {&::_pbi::kDescriptorMethods, &::descriptor_table_adb_5fknown_5fhosts_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace adb {
namespace proto {
class HostInfo::_Internal {
 public:
  using HasBits = decltype(::std::declval<HostInfo>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(HostInfo, _impl_._has_bits_);
};

constexpr HostInfo::ParseTableT_ HostInfo::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(HostInfo, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::adb::proto::HostInfo>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string guid = 1;
      {::_pbi::TcParser::FastUS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(HostInfo, _impl_.guid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // string guid = 1;
      {PROTOBUF_FIELD_OFFSET(HostInfo, _impl_.guid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    // no aux_entries
    {{
      "\22\4\0\0\0\0\0\0"
      "adb.proto.HostInfo"
      "guid"
    }},
  };
}


inline constexpr HostInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        guid_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()) {}

template <typename>
constexpr HostInfo::HostInfo(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL HostInfo::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) HostInfo(arena);
}
constexpr auto HostInfo::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(HostInfo), alignof(HostInfo));
}
constexpr auto HostInfo::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &HostInfo::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<HostInfo>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &HostInfo::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<HostInfo>(), &HostInfo::ByteSizeLong,
              &HostInfo::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(HostInfo, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_adb_5fknown_5fhosts_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct HostInfoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr HostInfoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 HostInfo_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(HostInfo::InternalGenerateClassData_(
            _default, &HostInfo_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<HostInfo>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~HostInfoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) HostInfo _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<HostInfo>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(HostInfoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST HostInfoGlobalsTypeInternal HostInfo_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* HostInfo_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return HostInfo_globals_.GetClassData();
#else
  return HostInfo_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class AdbKnownHosts::_Internal {
 public:
  using HasBits = decltype(::std::declval<AdbKnownHosts>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(AdbKnownHosts, _impl_._has_bits_);
};

constexpr AdbKnownHosts::ParseTableT_ AdbKnownHosts::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(AdbKnownHosts, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::GenericFallback,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::adb::proto::AdbKnownHosts>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated .adb.proto.HostInfo host_infos = 1;
      {::_pbi::TcParser::FastMtR1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(AdbKnownHosts, _impl_.host_infos_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated .adb.proto.HostInfo host_infos = 1;
      {PROTOBUF_FIELD_OFFSET(AdbKnownHosts, _impl_.host_infos_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::adb::proto::HostInfo>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::adb::proto::HostInfo_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr AdbKnownHosts::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        host_infos_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::adb::proto::AdbKnownHosts,
            PROTOBUF_FIELD_OFFSET(::adb::proto::AdbKnownHosts, _impl_.host_infos_)>()
         }
     {}

template <typename>
constexpr AdbKnownHosts::AdbKnownHosts(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : ::google::protobuf::Message(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL AdbKnownHosts::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) AdbKnownHosts(arena);
}
constexpr auto AdbKnownHosts::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(AdbKnownHosts), alignof(AdbKnownHosts));
}
constexpr auto AdbKnownHosts::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &AdbKnownHosts::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<AdbKnownHosts>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &AdbKnownHosts::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<AdbKnownHosts>(), &AdbKnownHosts::ByteSizeLong,
              &AdbKnownHosts::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(AdbKnownHosts, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_adb_5fknown_5fhosts_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AdbKnownHostsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AdbKnownHostsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 AdbKnownHosts_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(AdbKnownHosts::InternalGenerateClassData_(
            _default, &AdbKnownHosts_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<AdbKnownHosts>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AdbKnownHostsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) AdbKnownHosts _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<AdbKnownHosts>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AdbKnownHostsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AdbKnownHostsGlobalsTypeInternal AdbKnownHosts_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* AdbKnownHosts_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return AdbKnownHosts_globals_.GetClassData();
#else
  return AdbKnownHosts_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace proto
}  // namespace adb
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_adb_5fknown_5fhosts_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_adb_5fknown_5fhosts_2eproto = nullptr;
const ::uint32_t
    TableStruct_adb_5fknown_5fhosts_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::adb::proto::HostInfo, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::adb::proto::HostInfo, _impl_.guid_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::adb::proto::AdbKnownHosts, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::adb::proto::AdbKnownHosts, _impl_.host_infos_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::adb::proto::HostInfo)},
        {5, sizeof(::adb::proto::AdbKnownHosts)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::adb::proto::HostInfo_globals_,
        &::adb::proto::AdbKnownHosts_globals_,
};
const char descriptor_table_protodef_adb_5fknown_5fhosts_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\025adb_known_hosts.proto\022\tadb.proto\"\030\n\010Ho"
    "stInfo\022\014\n\004guid\030\001 \001(\t\"8\n\rAdbKnownHosts\022\'\n"
    "\nhost_infos\030\001 \003(\0132\023.adb.proto.HostInfoB3"
    "\n\035com.android.server.adb.protosB\022AdbKnow"
    "nHostsProtob\006proto3"
};
static ::absl::once_flag descriptor_table_adb_5fknown_5fhosts_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_adb_5fknown_5fhosts_2eproto = {
    false,
    false,
    179,
    descriptor_table_protodef_adb_5fknown_5fhosts_2eproto,
    "adb_known_hosts.proto",
    &descriptor_table_adb_5fknown_5fhosts_2eproto_once,
    nullptr,
    0,
    2,
    schemas,
    file_message_globals,
    TableStruct_adb_5fknown_5fhosts_2eproto::offsets,
    file_level_enum_descriptors_adb_5fknown_5fhosts_2eproto,
    file_level_service_descriptors_adb_5fknown_5fhosts_2eproto,
};
namespace adb {
namespace proto {
// ===================================================================

HostInfo::HostInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, HostInfo_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:adb.proto.HostInfo)
}
PROTOBUF_NDEBUG_INLINE HostInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::adb::proto::HostInfo& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        guid_(arena, from.guid_) {}

HostInfo::HostInfo(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const HostInfo& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, HostInfo_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  HostInfo* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:adb.proto.HostInfo)
}
PROTOBUF_NDEBUG_INLINE HostInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        guid_(arena) {}

inline void HostInfo::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
HostInfo::~HostInfo() {
  // @@protoc_insertion_point(destructor:adb.proto.HostInfo)
  SharedDtor(*this);
}
inline void HostInfo::SharedDtor(MessageLite& self) {
  HostInfo& this_ = static_cast<HostInfo&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.guid_.Destroy();
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull HostInfo_class_data_ =
        HostInfo::InternalGenerateClassData_(HostInfo_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HostInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&HostInfo_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(HostInfo_class_data_.tc_table);
  return HostInfo_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
HostInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&HostInfo_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&HostInfo_globals_));
  return HostInfo_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const HostInfo::ParseTableT_
    HostInfo::_table_ =
        HostInfo::InternalGenerateParseTable_(HostInfo_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void HostInfo::Clear() {
// @@protoc_insertion_point(message_clear_start:adb.proto.HostInfo)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.guid_.ClearNonDefaultToEmpty();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL HostInfo::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const HostInfo& this_ = static_cast<const HostInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL HostInfo::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const HostInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:adb.proto.HostInfo)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // string guid = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_guid().empty()) {
      const ::std::string& _s = this_._internal_guid();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "adb.proto.HostInfo.guid");
      target = stream->WriteStringMaybeAliased(1, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:adb.proto.HostInfo)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t HostInfo::ByteSizeLong(const MessageLite& base) {
  const HostInfo& this_ = static_cast<const HostInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t HostInfo::ByteSizeLong() const {
  const HostInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:adb.proto.HostInfo)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // string guid = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_guid().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_guid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void HostInfo::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<HostInfo*>(&to_msg);
  auto& from = static_cast<const HostInfo&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:adb.proto.HostInfo)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!from._internal_guid().empty()) {
      _this->_internal_set_guid(from._internal_guid());
    } else {
      if (_this->_impl_.guid_.IsDefault()) {
        _this->_internal_set_guid("");
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void HostInfo::CopyFrom(const HostInfo& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:adb.proto.HostInfo)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void HostInfo::InternalSwap(HostInfo* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.guid_, &other->_impl_.guid_, arena);
}

::google::protobuf::Metadata HostInfo::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

AdbKnownHosts::AdbKnownHosts(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AdbKnownHosts_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:adb.proto.AdbKnownHosts)
}
PROTOBUF_NDEBUG_INLINE AdbKnownHosts::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::adb::proto::AdbKnownHosts& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        host_infos_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::adb::proto::AdbKnownHosts,
              PROTOBUF_FIELD_OFFSET(::adb::proto::AdbKnownHosts, _impl_.host_infos_)>()
          , from.host_infos_
        }
     {}

AdbKnownHosts::AdbKnownHosts(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const AdbKnownHosts& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, AdbKnownHosts_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  AdbKnownHosts* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:adb.proto.AdbKnownHosts)
}
PROTOBUF_NDEBUG_INLINE AdbKnownHosts::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        host_infos_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::adb::proto::AdbKnownHosts,
            PROTOBUF_FIELD_OFFSET(::adb::proto::AdbKnownHosts, _impl_.host_infos_)>()
         }
     {}

inline void AdbKnownHosts::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
AdbKnownHosts::~AdbKnownHosts() {
  // @@protoc_insertion_point(destructor:adb.proto.AdbKnownHosts)
  SharedDtor(*this);
}
inline void AdbKnownHosts::SharedDtor(MessageLite& self) {
  AdbKnownHosts& this_ = static_cast<AdbKnownHosts&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull AdbKnownHosts_class_data_ =
        AdbKnownHosts::InternalGenerateClassData_(AdbKnownHosts_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AdbKnownHosts::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AdbKnownHosts_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(AdbKnownHosts_class_data_.tc_table);
  return AdbKnownHosts_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
AdbKnownHosts::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&AdbKnownHosts_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&AdbKnownHosts_globals_));
  return AdbKnownHosts_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const AdbKnownHosts::ParseTableT_
    AdbKnownHosts::_table_ =
        AdbKnownHosts::InternalGenerateParseTable_(AdbKnownHosts_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void AdbKnownHosts::Clear() {
// @@protoc_insertion_point(message_clear_start:adb.proto.AdbKnownHosts)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.host_infos_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL AdbKnownHosts::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const AdbKnownHosts& this_ = static_cast<const AdbKnownHosts&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL AdbKnownHosts::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const AdbKnownHosts& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:adb.proto.AdbKnownHosts)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .adb.proto.HostInfo host_infos = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_host_infos_size());
         i < n; i++) {
      const auto& repfield = this_._internal_host_infos().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:adb.proto.AdbKnownHosts)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t AdbKnownHosts::ByteSizeLong(const MessageLite& base) {
  const AdbKnownHosts& this_ = static_cast<const AdbKnownHosts&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t AdbKnownHosts::ByteSizeLong() const {
  const AdbKnownHosts& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:adb.proto.AdbKnownHosts)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .adb.proto.HostInfo host_infos = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_host_infos_size();
      for (const auto& msg : this_._internal_host_infos()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void AdbKnownHosts::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<AdbKnownHosts*>(&to_msg);
  auto& from = static_cast<const AdbKnownHosts&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:adb.proto.AdbKnownHosts)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_host_infos()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_host_infos());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void AdbKnownHosts::CopyFrom(const AdbKnownHosts& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:adb.proto.AdbKnownHosts)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void AdbKnownHosts::InternalSwap(AdbKnownHosts* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.host_infos_.InternalSwap(&other->_impl_.host_infos_);
}

::google::protobuf::Metadata AdbKnownHosts::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace proto
}  // namespace adb
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_adb_5fknown_5fhosts_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
